//
// ********************************************************************
// *                                                                  *
// *   Test Case for Geant4 Simulation:                               *
// *     Platinum Target 3cm dia  6cm long                            *
// *     30Gev Proton Beam  pointed at center of target               *
// *     Detect particle type and count at 16 degree angle            *
// *     at solid angle of 5 degree a distance of 14.5cm              *
// *     from beam contact point on target                            *
// *                                                                  *
// ********************************************************************
//
//
#include <stdlib.h>
//#include <vector>
//#include <fstream>
//#include <string>
//#include <sstream>

#include "TFile.h"
//#include "TH1.h"
//#include "TH2.h"
#include "hList.hh"
#include "TTree.h"

#include "G4SystemOfUnits.hh"

#include "myDetectorConstruction.hh"
#include "myPhysicsList.hh"
#include "myPrimaryGeneratorAction.hh"
#include "myRunAction.hh"
#include "myEventAction.hh"
#include "mySteppingAction.hh"
#include "mySteppingVerbose.hh"



//#include "LHEP.hh"
//#include "QGSP_BERT_TRV.hh"
//#include "QGSP_BIC.hh"
//#include "QGSP.hh"
////#include "QGSC.hh"
//#include "QGSC_BERT.hh"
////#include "LHEP_BERT.hh"
//#include "LHEP.hh"
#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"
#include "TRandom.h"
//ROOT LIBRARIES

#include "tripTreeStructs.hh"
#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#endif

#ifdef G4UI_USE
#include "G4UIExecutive.hh"
#endif




//#include "outTree.h"

struct mcDefaults_t {
  char inFileName[80];
  char outFileName[80];
  int nToGen,visModeUse;
  double xOffset,yOffset,foilZ;
  double thickness;
  char decayFile[80];
  double spotRadius;
  double seedVal;
  int mode;
  int axisAlign;
  double beamMomentum;
  double deadLow,deadHigh;
};

//FUNCTION PROTOTYPES
void printUsage(); // print usage
mcDefaults_t readMcDefaults(); //Read the default settings from mcDefaults.conf 
void readDecayFile(myPrimaryGeneratorAction* gen_action,char *inFile);//Read file given in command line or from mcDefaults.conf
G4String GetParticleName();//Get name from decay file
G4double GetELow();//Get minimum of energy range from decay file
G4double GetEHigh();//Get maximum of energy range from decay file

// ********************************************************************
//--> Histos    
//TH2D *hErVsTheta;
//TH2D *hErVsThetaFull;
//TH1D *hFrac;
//TH1D *hAsymmetry0;
//TH1D *hAsymmetry1;
//TH1D *hAsymmetry2;
//TH1D *hAsymmetryDet1;
//TH1D *hAsymmetryDet2;
//TH2D *h2d401a;
//TH1D *hDeDx1;
//TH1D *hDeDx2;
//TH2D *hEvsZ;
//TH1D *hEZ;
//TH1D *hHit;
//TH1D *hHitSecondary;
//TH1D *hDecayTest;
//TH1D *heDepNwTh;
//TH1D *heDepNw;
//TH2D *heDepNwMult;
//TH1D *heDepNwSmear12;
//TH1D *heDepNwSmear20;
//TH1D *heDepNwPart;
//TH1D *heDepNwSmear12Part;
//TH1D *heDepNwThPart;
//TH1D *heDepNwSmear20Part;
//TH1D *heDepNwSmear12PartAlpha;
//TH1D *heDepNwSmear12Alpha;
//TH2D *heDepNwSmear12AlphaSec;
//TH1D *hElectronKe;
//TH2D *hElectronThetaKe;
//TH1D *hDeltaRayKe;
//TH2D *hDeltaRayThetaKe;
//TH2D *hXY;
//TH2D *hDeltaThetaPhi;
//TH2D *hDeltaThetaKEr;
//TH2D *hDeltaPhiKEr;
//TH1D *hNRecoil;
//TH1D *hNDeltaRay;
//TH1D *hNBoth;
//TH2D *hBeamXY;
TFile* inFile;
TFile* inFile2;
TTree* t1;

mcHistos_t myHistos;

int main(int argc,char** argv)
{
  rxnDecay_t myDecay;
  char outFileName[80],inFileName[80],inFileName2[80], decayFile[80];
  int nToGen,visModeUse;
  double thickness;
  double spotRadius;
  double deadLow,deadHigh;
  double beamMomentum;
  double seedVal;
  double mode;
  double xOffset,yOffset;
  double  foilZ;
  int    axisAlign;
  char   *argptr;
  //Read the defaults
  mcDefaults_t mcDefaults = readMcDefaults(); 
  //Set the defaults
  nToGen       = mcDefaults.nToGen;
  thickness    = mcDefaults.thickness;
  seedVal      = mcDefaults.seedVal;
  mode         = mcDefaults.mode;
  spotRadius   = mcDefaults.spotRadius;
  deadLow      = mcDefaults.deadLow;
  deadHigh     = mcDefaults.deadHigh;
  xOffset      = mcDefaults.xOffset; 
  yOffset      = mcDefaults.yOffset; 
  foilZ        = mcDefaults.foilZ;
  axisAlign    = mcDefaults.axisAlign; 
  beamMomentum = mcDefaults.beamMomentum;
  visModeUse   = mcDefaults.visModeUse;
  sprintf(outFileName,mcDefaults.outFileName);
  sprintf(inFileName2,"./cobrems.root");
  sprintf(inFileName,mcDefaults.inFileName);
  sprintf(decayFile,mcDefaults.decayFile);

  for (int i=1; i<argc; i++) {
    argptr = argv[i];
    if (*argptr == '-') {
      argptr++;
      switch (*argptr) {
      case 'h':
        printUsage();
        break;
      case 'i':
        strcpy(inFileName,++argptr);
        break;
      case 's':
        seedVal = atoi(++argptr);
        break;
      case 'm':
        mode = atoi(++argptr);
        break;
      case 'd':
        strcpy(decayFile,++argptr);
        break;
      case 't':
        thickness = atof(++argptr);
        break;
      case 'z':
        foilZ = atof(++argptr);
        break;
      case 'x':
        xOffset = atof(++argptr);
        break;
      case 'y':
        yOffset = atof(++argptr);
        break;
      case 'a':
        axisAlign = atoi(++argptr);
        break;
      case 'p':
        beamMomentum = atof(++argptr);
        break;
      case 'n':
        nToGen = atoi(++argptr);
        break;
      case 'v':
        visModeUse = atoi(++argptr);
        break;
      case 'o':
        strcpy(outFileName,++argptr);
        break;
      case 'r':
	spotRadius = atof(++argptr);
        break;
      case 'l':
	deadLow = atof(++argptr);
        break;
      case 'u':
	deadHigh = atof(++argptr);
        break;
      default:
        fprintf(stderr,"Unrecognized argument: [-%s]\n\n",argptr);
        printUsage();
        break;
      }
    }
  }

  //Setup the output tree                                                                 
  TTree *outTree = new TTree("tpolTree1","tpol1");
  tpol1_t tpol1; //tpol1_t defined in outTree.h         
  //setBranchesTpol1(outTree,&tpol1); //setBranchesTpol1 defined in outTree.h

  outTree->Branch("nSectorHits",&tpol1.nSectorHits,"nSectorHits/I");
  outTree->Branch("mcSector",&tpol1.mcSector,"mcSector[nSectorHits]/I");
  outTree->Branch("phi",tpol1.phi,"phi[nSectorHits]/D");                                               
  outTree->Branch("eDep",tpol1.eDep,"eDep[nSectorHits]/D");
  outTree->Branch("eRIn",&tpol1.eRIn,"eRIn/D");
  outTree->Branch("mPair",&tpol1.mPair,"mPair/D");
  outTree->Branch("mPair2",&tpol1.mPair2,"mPair2/D");
  outTree->Branch("cosR",&tpol1.cosR,"cosR/D");
  outTree->Branch("beamX",&tpol1.beamX,"beamX/D");
  outTree->Branch("beamY",&tpol1.beamY,"beamY/D");
  outTree->Branch("ePos",&tpol1.ePos,"ePos/D");
  outTree->Branch("eNeg",&tpol1.eNeg,"eNeg/D");
  outTree->Branch("weightXS",&tpol1.weightXS,"weightXS/D");
  outTree->Branch("eGamma",&tpol1.eGamma,"eGamma/D");

  inFile=new TFile(inFileName);
  t1=(TTree*)inFile->Get("t1");
  if (t1->GetEntries() < nToGen) nToGen = t1->GetEntries();
  //Define Histogram binning and ranges
  myHistos.hDeDx1 = new TH1D("hDeDx1","",1000,0.0,20.0); myHistos.hDeDx1->Sumw2();
  myHistos.hDeDx2 = new TH1D("hDeDx2","",1000,0.0,20.0); myHistos.hDeDx2->Sumw2();
  myHistos.hEvsZ = new TH2D("hEvsZ","",1000,0.0,10.0,1000,0.0,20.0); myHistos.hEvsZ->Sumw2();
  myHistos.hEZ = new TH1D("hEZ","",100,0.0,10.0); myHistos.hEZ->Sumw2();
  myHistos.hHit = new TH1D("hHit","",300,5.0,3005); myHistos.hHit->Sumw2();
  myHistos.hDecayTest = new TH1D("hDecayTest","",1000,0.0,1.0); myHistos.hDecayTest->Sumw2();
  myHistos.heDepNw = new TH1D("heDepNw","",1000,0.0,1.0); myHistos.heDepNw->Sumw2();
  myHistos.heDepNwMult = new TH2D("heDepNwMult","",11,-0.5,10.5,1000,0.0,1.0); 
  myHistos.heDepNwTh = new TH1D("heDepNwTh","",2000,0.0,2.0); myHistos.heDepNwTh->Sumw2();
  myHistos.heDepNwSmear20 = new TH1D("heDepNwSmear20","",1000,0.0,1.0); myHistos.heDepNwSmear20->Sumw2();
  myHistos.heDepNwSmear12 = new TH1D("heDepNwSmear12","",1000,0.0,1.0); myHistos.heDepNwSmear12->Sumw2();
  myHistos.heDepNwPart = new TH1D("heDepNwPart","",2000,0.0,2.0); myHistos.heDepNwPart->Sumw2();
  myHistos.heDepNwSmear20Part = new TH1D("heDepNwSmear20Part","",1000,0.0,1.0); myHistos.heDepNwSmear20Part->Sumw2();
  myHistos.heDepNwThPart = new TH1D("heDepNwThPart","",2000,0.0,2.0); myHistos.heDepNwThPart->Sumw2();
  myHistos.heDepNwSmear12Part = new TH1D("heDepNwSmear12Part","",1000,0.0,1.0); myHistos.heDepNwSmear12Part->Sumw2();
  myHistos.hHitSecondary = new TH1D("hHitSecondary","",300,5.0,3005); myHistos.hHitSecondary->Sumw2();
  myHistos.heDepNwSmear12PartAlpha = new TH1D("heDepNwSmear12PartAlpha","",1000,5.0,6.0); myHistos.heDepNwSmear12PartAlpha->Sumw2();
  myHistos.heDepNwSmear12Alpha = new TH1D("heDepNwSmear12Alpha","",3000,3.0,6.0); myHistos.heDepNwSmear12Alpha->Sumw2();
  myHistos.heDepNwSmear12AlphaSec = new TH2D("heDepNwSmear12AlphaSec","",3000,3.0,6.0,32,0.5,32.5); myHistos.heDepNwSmear12AlphaSec->Sumw2();
  myHistos.hBeamXY = new TH2D("hBeamXY","",81,-4.05,4.05,81,-4.05,4.05);
  myHistos.hFrac    = new TH1D("hFrac", "", 200, -1.0, 1.0); myHistos.hFrac->Sumw2(); 
  myHistos.hAsymmetry0    = new TH1D("hAsymmetry0", "", 32, -180.0, 180.0); myHistos.hAsymmetry0->Sumw2(); 
  myHistos.hAsymmetry1    = new TH1D("hAsymmetry1", "", 32, -180.0, 180.0); myHistos.hAsymmetry1->Sumw2(); 
  myHistos.hAsymmetry2    = new TH1D("hAsymmetry2", "", 32, -180.0, 180.0); myHistos.hAsymmetry2->Sumw2(); 
  myHistos.hAsymmetryDet1 = new TH1D("hAsymmetryDet1", "", 32, 0, 360); myHistos.hAsymmetryDet1->Sumw2();
  myHistos.hAsymmetryDet2 = new TH1D("hAsymmetryDet2", "", 32, 0, 360); myHistos.hAsymmetryDet2->Sumw2();

  myHistos.hElectronKe = new TH1D("hElectronKe","",100,0.0,10.0); myHistos.hElectronKe->Sumw2();
  myHistos.hDeltaRayKe = new TH1D("hDeltaRayKe","",100,0.0,10.0); myHistos.hDeltaRayKe->Sumw2();

  myHistos.hElectronThetaKe = new TH2D("hElectronThetaKe","",60,0.0,60.0,2000,0.0,20.0); 
  myHistos.hDeltaRayThetaKe = new TH2D("hDeltaRayThetaKe","",60,0.0,60.0,2000,0.0,20.0); 
  myHistos.hXY = new TH2D("hXY","",100,-50.0,50.0,100,-50.0,50.0); 
  myHistos.hNDeltaRay = new TH1D("hNDeltaRay","",6,-0.5,5.5); myHistos.hNDeltaRay->Sumw2();
  myHistos.hNRecoil = new TH1D("hNRecoil","",6,-0.5,5.5); myHistos.hNRecoil->Sumw2();
  myHistos.hNBoth = new TH1D("hNBoth","",6,-0.5,5.5); myHistos.hNBoth->Sumw2();
  myHistos.hDeltaThetaPhi = new TH2D("hDeltaThetaPhi","",100,-50.0,50.0,100,-50.0,50.0); 
  myHistos.hDeltaThetaKEr = new TH2D("hDeltaThetaKEr","",100,-50.0,50.0,2000,0.0,20.0); 
  myHistos.hDeltaPhiKEr = new TH2D("hDeltaPhiKEr","",100,-50.0,50.0,2000,0.0,20.0); 

  TFile *fout = new TFile(outFileName,"RECREATE");  //DEFINE OUTPUT ROOT FILE 
  // Run manager
  G4RunManager * runManager = new G4RunManager;

  G4cout<<"test:1"<<G4endl;

  // User Initialization classes (mandatory)
  myDetectorConstruction* detector = new myDetectorConstruction;
  detector->SetConverterThickness(thickness*um);//Set converter thickness to what is passed by command line or from mcDefaults
  detector->SetFoilZ(foilZ*mm);
  runManager->SetUserInitialization(detector);

  G4cout<<"test:2"<<G4endl;

  G4VUserPhysicsList* physics = new myPhysicsList;
  runManager->SetUserInitialization(physics);

  G4cout<<"test:3"<<G4endl;

  // User Action classes
  myRunAction* run_action = new myRunAction;
  runManager->SetUserAction(run_action);
  run_action->SetSeedVal(seedVal);//Set random seed value as defined through command line or mcDefaults

  G4cout<<"test:4"<<G4endl;

  myPrimaryGeneratorAction* gen_action = new myPrimaryGeneratorAction(detector);
  runManager->SetUserAction(gen_action);
  gen_action->SetBeamMomentum(beamMomentum*MeV);//Set momentum of beam as defined through command line or mcDefaults
  gen_action->SetSpotRadius(spotRadius*mm);//Set radius of beam spot as defined through command line or mcDefaults
  gen_action->SetMode(mode);
  gen_action->SetBeamX(xOffset*mm);
  gen_action->SetBeamY(yOffset*mm);
  gen_action->SetAxisAlign(axisAlign);
  G4cout<<"xoff, yoff = "<<xOffset<<" , "<<yOffset<<G4endl;
  readDecayFile(gen_action,decayFile);//Read decay file as defined through command line or mcDefaults
  G4cout<<"test:5"<<G4endl;

  myEventAction* event_action = new myEventAction(gen_action);
  runManager->SetUserAction(event_action);
  event_action->SetOutTree(outTree);
  event_action->SetOutTreeStruct(&tpol1);

  G4cout<<"test:6"<<G4endl;
  //Set the generator tree pointer
  gen_action->SetGenTree(t1);
  mySteppingAction* stepping_action = new mySteppingAction(detector, event_action, gen_action);
  runManager->SetUserAction(stepping_action);

  //G4double  deadLow = 400*um;
  //G4double  deadHigh = 400*um;
  stepping_action->SetDeadLow(deadLow*um);
  stepping_action->SetDeadHigh(deadHigh*um);
  G4cout<<"test:7"<<G4endl;
  // Initialize G4 kernel
  G4UImanager* UI = G4UImanager::GetUIpointer();

  //UI->ApplyCommand("/tracking/verbose 1");//For verbose tracking information
  UI->ApplyCommand("/tracking/verbose 0"); 
  UI->ApplyCommand("/event/verbose 0");
  //UI->ApplyCommand("/event/verbose 1"); //For verbose event information
  runManager->Initialize();

  G4cout<<"test:8"<<G4endl;

  G4UImanager* UImanager = G4UImanager::GetUIpointer();

  if (visModeUse != 1)   // batch mode
    {
      G4String command = "/control/execute ";
      runManager->BeamOn(nToGen); //Number of events to generate
    }
  else
    {  // interactive mode : define UI session
      
#ifdef G4VIS_USE
      //Initialize visualization                                                   
      G4VisManager* visManager = new G4VisExecutive;
      visManager->Initialize();
#endif
      
#ifdef G4UI_USE
      G4UIExecutive* ui = new G4UIExecutive(argc, argv);
#ifdef G4VIS_USE
      UImanager->ApplyCommand("/control/execute vis.mac");
#endif
      if (ui->IsGUI())
	UImanager->ApplyCommand("/control/execute visTutor/gui.mac");
      ui->SessionStart();
      delete ui;
#endif
    }
  G4cout<<"test:9"<<G4endl;
  delete runManager;
  
  fout->cd();
  //hDecayTest->Write();
  //heDepNwTh->Write();
  //heDepNw->Write();
  //heDepNwMult->Write();
  //heDepNwSmear12->Write();
  //heDepNwSmear20->Write();
  //heDepNwPart->Write();
  //heDepNwSmear12Part->Write();
  //heDepNwThPart->Write();
  //heDepNwSmear20Part->Write();
  //heDepNwSmear12Alpha->Write();
  //heDepNwSmear12AlphaSec->Write();
  //heDepNwSmear12PartAlpha->Write();
  //hDeDx1->Write();
  //hDeDx2->Write();
  //hFrac->Write();
  //hBeamXY->Write();
  //hAsymmetry0->Write();
  //hAsymmetry1->Write();
  //hAsymmetry2->Write();
  //hAsymmetryDet1->Write();
  //hAsymmetryDet2->Write();
  myHistos.hElectronKe->Write();
  myHistos.hDeltaRayKe->Write();
  myHistos.hElectronThetaKe->Write();
  myHistos.hDeltaRayThetaKe->Write();
  //hXY->Write();
  //hNRecoil->Write();
  //hNDeltaRay->Write();
  //hNBoth->Write();
  //hDeltaThetaPhi->Write();
  //hDeltaThetaKEr->Write();
  //hDeltaPhiKEr->Write();
  outTree->Write();

  fout->Close();
  return 0;
}

//Prints command line switch information
void printUsage(){
  fprintf(stderr,"\nSWITCHES:\n");
  fprintf(stderr,"-h\tPrint this message\n");
  fprintf(stderr,"-i<arg>\tInFile name\n");
  fprintf(stderr,"-n<arg>\tNumber of events to generate\n");
  fprintf(stderr,"-t<arg>\tThickness in microns\n");
  fprintf(stderr,"-z<arg>\tfoil Z in mm\n");
  fprintf(stderr,"-s<arg>\tRandom seed number\n");
  fprintf(stderr,"-m<arg>\tmode (1 = Recoil Only, 2 = Pair Only, 3 = Pair and Recoil, 4 = Source)\n");
  fprintf(stderr,"-d<arg>\tDecay file name\n");
  fprintf(stderr,"-a<arg>\tAxis alignment (0 = Flip to make amorphous, 1 = As is, 2 = Switch x and y axis)\n");
  fprintf(stderr,"-r<arg>\tBeam Radius in mm\n");
  fprintf(stderr,"-x<arg>\tBeam x-offset in mm\n");
  fprintf(stderr,"-y<arg>\tBeam y-offset in mm\n");
  fprintf(stderr,"-l<arg>\tDead region from face of detector um\n");
  fprintf(stderr,"-u<arg>\tDead region from back of detector um\n");
  fprintf(stderr,"-p<arg>\tBeam Momentum in MeV\n");
  fprintf(stderr,"-o<arg>\tOutFile name\n");
  fprintf(stderr,"-v<arg>\tVisualizaton mode if set to 1\n");

  G4cout<<""<<G4endl;
  G4cout<<"The above switches overide the default setting."<<G4endl;
  G4cout<<"The default settings are found in the file mcDefaults.conf"<<G4endl;
  G4cout<<"The user should modify the vsDefault.conf file to suit their taste"<<G4endl;

  G4cout<<""<<G4endl;
  mcDefaults_t mcDefaults = readMcDefaults(); //Read the defaults

  G4cout<<"The current default operation is equivalent to the command:"<<G4endl;
  G4cout<<"my"
	<<" -n"<<mcDefaults.nToGen
	<<" -t"<<mcDefaults.thickness
	<<" -z"<<mcDefaults.foilZ
	<<" -s"<<mcDefaults.seedVal
	<<" -m"<<mcDefaults.mode
	<<" -a"<<mcDefaults.axisAlign
	<<" -x"<<mcDefaults.xOffset
	<<" -y"<<mcDefaults.yOffset
	<<" -r"<<mcDefaults.spotRadius
	<<" -l"<<mcDefaults.deadLow
	<<" -u"<<mcDefaults.deadHigh
    	<<" -d"<<mcDefaults.decayFile
	<<" -p"<<mcDefaults.beamMomentum
	<<" -o"<<mcDefaults.outFileName
	<<" -v"<<mcDefaults.visModeUse
        <<" -i"<<mcDefaults.inFileName
	<<"\n"<<G4endl;

  exit(0);
}
//Reads Default file
mcDefaults_t readMcDefaults(){
  mcDefaults_t mcDefaults;
  char *fToRead;
  char parType[80];
  fToRead = "./mcDefaults.conf";
  std::string line;
  std::stringstream os(line);
  std::string temp;
  ifstream myfile (fToRead);
  int varI=0,emptyLine;
  double varD = 0.0;
  char varC[80];
  if (myfile.is_open())
    {
      while ( myfile.good() )
        {
          int i = 0;
          getline(myfile,line);
	  std::stringstream os(line);
          emptyLine = 1;
          while (os >> temp) {
            emptyLine = 0;
            if (i==0){
              sprintf(parType,temp.c_str());
            } else {
              if (i==1) varI = atoi(temp.c_str());
              if (i==1) varD = atof(temp.c_str());
              if (i==1) sprintf(varC,temp.c_str());
            }
            i++;
          }
          if (emptyLine==0){
            if (strcmp(parType,"outFileName") == 0) sprintf(mcDefaults.outFileName,varC);
            if (strcmp(parType,"inFileName") == 0) sprintf(mcDefaults.inFileName,varC);
	    if (strcmp(parType,"decayFile") == 0) sprintf(mcDefaults.decayFile,varC);
	    if (strcmp(parType,"nToGen") == 0) mcDefaults.nToGen = varI;
            if (strcmp(parType,"thickness") == 0) mcDefaults.thickness = varD;
            if (strcmp(parType,"foilZ") == 0) mcDefaults.foilZ = varD;
	    if (strcmp(parType,"seedVal") == 0) mcDefaults.seedVal = varD;
	    if (strcmp(parType,"mode") == 0) mcDefaults.mode = varI;
            if (strcmp(parType,"xOffset") == 0) mcDefaults.xOffset = varD;
            if (strcmp(parType,"yOffset") == 0) mcDefaults.yOffset = varD;
            if (strcmp(parType,"axisAlign") == 0) mcDefaults.axisAlign = varI;
	    if (strcmp(parType,"spotRadius") == 0) mcDefaults.spotRadius = varD;
	    if (strcmp(parType,"deadLow") == 0) mcDefaults.deadLow = varD;
	    if (strcmp(parType,"deadHigh") == 0) mcDefaults.deadHigh = varD;
	    if (strcmp(parType,"beamMomentum") == 0) mcDefaults.beamMomentum = varD;
	    if (strcmp(parType,"visModeUse") == 0) mcDefaults.visModeUse = varI;
          }
        }
      myfile.close();
    }
  else G4cout << "!!!!!!!!!!!Unable to open mcDefaults.conf!!!!!!!!!!";
  return mcDefaults;  
}
//Reads Decay file 
void readDecayFile(myPrimaryGeneratorAction* gen_action,char *decayFile){
  std::string myLineElement; 
  std::string line;
  char particleName[80],element1[80]; 
  double eLow,eHigh,perHundred;
  rxnDecay_t myDecay;
  ifstream myfile (decayFile);//Open the file
  eLow  = 0.0;
  eHigh = 0.0;
  perHundred = 0.0;
  if (myfile.is_open()){
    while ( myfile.good() ){
      bool singleEnergy = false;
      getline(myfile,line);
      std::stringstream lineStream(line);
      //sprintf(element1,"");
      int i = 1; //Start counting line elements at zero
      while (lineStream >> myLineElement) {
	if (i==1) sprintf(element1,myLineElement.c_str());    //Element 1 Decay or other (Decay means read line)
	if (i==2) sprintf(particleName,myLineElement.c_str());//Element 2 is particle name
	if (i==3) eLow       = atof(myLineElement.c_str());   //Element 3 is minimum of energy range
	if (i==4) eHigh      = atof(myLineElement.c_str());   //Element 4 is maximum of energy range
	if (i==5) perHundred = atof(myLineElement.c_str());   //Element 5 is number of decays per 100 events
	i++;	
      }
      if (strcmp(element1,"Decay") == 0){//If the first element says "Decay", read line and define variables as follows
	if (eLow == eHigh) singleEnergy = true;
	myDecay.particleName = particleName;
	myDecay.singleEnergy = singleEnergy;
	myDecay.eLow = eLow*keV;
	myDecay.eHigh = eHigh*keV;
 	myDecay.decayPer100 = perHundred;
	gen_action->SetDecay(myDecay);
      }
    }
    myfile.close();
  }
}
