//
// ********************************************************************
// *                                                                  *
// *                                                                  *
// *                                                                  *
// ********************************************************************
//

#include <vector>
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "globals.hh"
#include "G4VUserDetectorConstruction.hh"
#include "G4SubtractionSolid.hh"
#include "G4UnionSolid.hh"
//#include "G4Box.hh"
//#include "G4Tubs.hh"
//#include "G4Sphere.hh"
//#include "G4LogicalVolume.hh"
//#include "G4PVPlacement.hh"

#include "G4SystemOfUnits.hh"



class G4Box;
class G4Tubs;
class G4Sphere;
class G4LogicalVolume;
class G4VPhysicalVolume;
class G4Material;
class G4VPVParameterisation;
class G4UserLimits;
class myDetectorMessenger;

// ********************************************************************

using namespace std;

class myDetectorConstruction : public G4VUserDetectorConstruction
{
public:
  
  myDetectorConstruction();
  ~myDetectorConstruction();
  
public:
  
  G4VPhysicalVolume* Construct();

public:  
  const G4VPhysicalVolume* GetphysiWorld()  {return physiWorld;};           
  //const G4VPhysicalVolume* GetConverter()   {return physiConverter;};
  G4double  GetConverterThickness() {return thickness;}
  void  SetConverterThickness(G4double cVal) {thickness = cVal;}
  G4double  GetFoilZ() {return foilZ;}
  void  SetFoilZ(G4double zVal) {foilZ = zVal;}
  void  SetRingSectorID(G4String);
  G4int GetRingID(){return triPolRingID;};
  G4int GetSectorID(){return triPolSectorID;};
  G4double GetZPolFace(){return zPolFace;};
  G4double GetZPolBack(){return zPolBack;};
  G4ThreeVector GetPosFoil(){return posFoil;};
  void  MakeShapes();
  void  MakeTube();
  void  MakeBoxes();

private:

  G4LogicalVolume*   logicBlank;
  G4VPhysicalVolume* physiBlank;
  G4Tubs* solidBlank;

  G4LogicalVolume*   logicBlank2;
  G4VPhysicalVolume* physiBlank2;
  G4Tubs* solidBlank2;

  G4LogicalVolume*   logicBlank3;
  G4VPhysicalVolume* physiBlank3;
  G4Tubs* solidBlank3;

  G4LogicalVolume*   logicBlank4;
  G4VPhysicalVolume* physiBlank4;
  G4Tubs* solidBlank4;

  G4LogicalVolume*   logicBlank5;
  G4VPhysicalVolume* physiBlank5;
  G4Tubs* solidBlank5;

  G4Box*             solidWorld;     // pointer to the solid envelope 
  G4LogicalVolume*   logicWorld;     // pointer to the logical envelope
  G4VPhysicalVolume* physiWorld;     // pointer to the physical envelope

  G4VPhysicalVolume *physiCardS3;
  G4VPhysicalVolume *physiBoxWithCutout;
  G4VPhysicalVolume *physiDoorFlange;
  G4VPhysicalVolume *physiRailGuide;
  G4VPhysicalVolume *physiConverter;
  G4VPhysicalVolume *physiFoil;
  G4VPhysicalVolume *physiMotor;

  G4double thickness,zPolFace,zPolBack,foilZ;
  G4ThreeVector posFoil;  
  G4int triPolRingID,triPolSectorID;
  
  //Detector Geometry
  G4double doorX,doorFlangeX,doorY,doorZ;
  G4ThreeVector posTest;
  G4double inch;
  //Door
  G4Box *solidDoor;
  G4LogicalVolume *logicDoor;
  G4VPhysicalVolume *physiDoor;
  
  
  int indexCube1,indexCube2, indexDoor, indexDoorFlange, indexRailGuide, indexRailGuideThin, indexRailGuideBox, indexMotorPlate, indexConverter, indexConverterSub, indexMotorTube, indexMotorGear, indexBeamHole, indexFeedHole, indexTurboHole, indexMotorHole, indexAuxHole, indexConverterHole;
  int indexBeamPipeR, indexBeamPipeL, indexFeedPipe, indexMotorPipe, indexTurboPipe;
  int indexAuxPipeL,indexAuxPipeR;
  int indexHRackL, indexVRackL , indexHRackR , indexVRackR , indexHRackB , indexVRackB , indexHBarT , indexHBarB , indexVSupportF , indexVSupportB , indexHSupportF , indexHSupportB , indexLegF , indexLegB, indexPlate;
  int indexBeamFlangeR, indexBeamFlangeL, indexAuxFlangeR, indexAuxFlangeL, indexTurboFlange, indexMotorFlange, indexFeedFlange;
  int indexAuxCapL ,indexAuxCapR, indexMotorCap, indexFeedCap;
  
G4LogicalVolume *logicCube1;
  G4LogicalVolume *logicCube2;
  
  vector <G4Box*> solidShape;
  vector <G4Tubs*> solidTube;
  vector <G4LogicalVolume*> logicTube;
  vector <G4VPhysicalVolume*> physiTube;
  vector <G4Box*> solidRect;
  vector <G4LogicalVolume*> logicRect;
  vector <G4VPhysicalVolume*> physiRect;
  vector <G4double> tubeLength;
  vector <G4Tubs*> solidCap;
  vector <G4LogicalVolume*> logicCap;
  vector <G4VPhysicalVolume*> physiCap;
  vector <G4Tubs*> solidAddTube;
  vector <G4double> holeDim;
  vector <G4SubtractionSolid*> solidBoxHoles;
  vector <G4LogicalVolume*> logicAdd;
  vector <G4VPhysicalVolume*> physiAdd;
};  

