//
// ********************************************************************
// *                                                                  *
// *                                                                  *
// *                                                                  *
// ********************************************************************
//
 


#include "TTree.h"
#include "TLorentzVector.h"

#include <vector>
//#include "outTree.h"
#include "G4VUserPrimaryGeneratorAction.hh"
#include "G4SystemOfUnits.hh"
#include "G4ThreeVector.hh"
#include "globals.hh"

class myDetectorConstruction;
class G4ParticleGun;
class G4Event;

struct rxnDecay_t {
  G4String particleName;
  bool singleEnergy;
  G4double eLow,eHigh;
  double decayPer100;
};

// ********************************************************************
 
class myPrimaryGeneratorAction : public G4VUserPrimaryGeneratorAction
{
public:
  myPrimaryGeneratorAction(myDetectorConstruction*);    
  ~myPrimaryGeneratorAction();
  
public:
  void GeneratePrimaries(G4Event*);
  void SetGenTree(TTree*);
  void SetWeightXSpointer(double*);
  void SetDeltaE12pointer(double*);

  void  SetMode(G4int modeTmp) {mode = modeTmp;};

  G4double GetThetaR(){return thetaR;};
  G4double GetPhiR(){return phiR;};
  G4double GetMoR(){return moR;};
  G4double GetKeR(){return keR;};
  G4double GetERIn(){return eR;};
  G4double GetMpair(){return mPair;};
  G4double GetCosR(){return cosR;};
  G4double GetMpair2(){return mPair2;};
  G4double GetWeightXS(){return weightXS;};
  G4double GetEgamma(){return eGamma;};
  G4double GetDeltaE12(){return deltaE12;};
  void  SetBeamMomentum(G4double pVal) {beamMomentum = pVal;}
  void  SetDecay(rxnDecay_t myDecayTmp) {myDecayVec.push_back(myDecayTmp);}
  void  SetConverterZ(G4double zVal) {converterZ = zVal;}
  void  SetBeamType(G4int bVal) {beamType = bVal;}
  void  SetBeamX(G4double xVal) {xOffset0 = xVal;}
  void  SetBeamY(G4double yVal) {yOffset0 = yVal;}
  void  SetSpotRadius(G4double sVal) {spotRadius = sVal;}
  void  SetAxisAlign(G4int axisAlignIn) {axisAlign = axisAlignIn;}
  G4double GetSpotRadius(){return spotRadius;};
  void  SetEventsToSkip(G4int skipVal) {nToSkip = skipVal;}
  G4double GetDePair();
  G4double GetEpos(){return posVec.E()*GeV;};
  G4double GetEneg(){return negVec.E()*GeV;};
  G4double GetBeamMomentum(){return beamMomentum;};
  G4ThreeVector GetVertex(){return vertex;};
  G4ThreeVector GetVertexTPOLCenter(){return vertexTPOLCenter;};
  G4ThreeVector GetRecoilVec(){return recVec3;};
  bool InPSCPeak(){
    G4double e1Val = posVec.E()*GeV;
    G4double e2Val = negVec.E()*GeV;
    bool inPSCPeak = false;
    bool inPeak    = false;
    bool e1InPSC   = false;
    bool e2InPSC   = false;
    bool inPSC     = false;
    double e12Val  = e1Val + e2Val;
    if (e1Val/GeV >= 1.1 && e1Val/GeV <= 2.35) e1InPSC = true;
    if (e2Val/GeV >= 1.1 && e2Val/GeV <= 2.35) e2InPSC = true;
    if (e1InPSC == true && e2InPSC == true) inPSC = true;
    if (e12Val/GeV >= 2.5 && e12Val/GeV <= 3.12) inPeak = true;
    if (inPSC == true && inPeak == true) inPSCPeak = true; 
    return inPSCPeak;
  }

private:
  G4int mode, axisAlign;
  G4ThreeVector vertex,recVec3,vertexTPOLCenter;
  TLorentzVector posVec,negVec;
  G4ParticleGun* particleGun;
  myDetectorConstruction* myDetector;
  TTree* genTree;
  G4int nToSkip;
  G4double thetaR,keR,moR,phiR,deltaE12,weightXS,eGamma;
  G4double moE,mE,beamMomentum,eR,mPair,mPair2;
  double cosR;
  G4double converterZ,beamType;
  G4double xOffset0,yOffset0,spotRadius,beamPosX,beamPosY;
  myDetectorConstruction* detector;
  char decayFile[80];
  //rxnDecay_t myDecayVec;
  std::vector <rxnDecay_t> myDecayVec;
  //rxnDecay_t testme;

    
  struct rxnShot_t {
    //Needed to fire the gun
    G4String particleName;
    G4double energy;
    G4ThreeVector momentum;
    G4ThreeVector position;
 };
 
};



// ********************************************************************



