//                                                                                                                            
// ********************************************************************                                                       
// *                                                                  *                                                       
// * Physical Target and Detector Definitions                         *                                                       
// *                                                                  *                                                       
// ********************************************************************                                                       
//                                                                                                                            

#include "myDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4MultiFunctionalDetector.hh"
#include "G4VPrimitiveScorer.hh"
#include "G4PSSphereSurfaceCurrent.hh"
#include "G4SDParticleFilter.hh"
#include "G4SDManager.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4ios.hh"
#include "G4NistManager.hh"
#include <stdio.h>
#include "G4UniformMagField.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"
#include "G4SubtractionSolid.hh"


// ********************************************************************                                                       

myDetectorConstruction::myDetectorConstruction() { }

// ********************************************************************                                                       

myDetectorConstruction::~myDetectorConstruction() { }

// ********************************************************************                 
G4VPhysicalVolume* myDetectorConstruction::Construct()
{
  inch                 = 2.54*cm;
  //distConvSSD = 33.7*mm;
  //========Material definition
  
  G4double density;
  
  G4NistManager* man = G4NistManager::Instance();
  
  G4Material* Silicon       = man->FindOrBuildMaterial("G4_Si");
  G4Material* Beryllium     = man->FindOrBuildMaterial("G4_Be");
  G4Material* Iron          = man->FindOrBuildMaterial("G4_Fe");
  G4Material* Aluminum      = man->FindOrBuildMaterial("G4_Al");
  G4Material* Polycarbonate = man->FindOrBuildMaterial("G4_POLYCARBONATE");

  G4double atomicNumber = 1.;
  G4double massOfMole   = 1.008*g/mole;
  density               = 1.e-25*g/cm3;
  G4double temperature  = 2.73*kelvin;
  
  //G4double pressure=3.e-18*pascal; //SPACE                                                                                  
  G4double pressure=0.0005*pascal;//10^-5 Torr
  G4Material *Vacuum =  new G4Material("Vacuum",atomicNumber,massOfMole,density,kStateGas,temperature,pressure);

  //ASDF
  //Silicon = Vacuum;
  //Beryllium = Vacuum;
  //Iron = Vacuum;
  //Aluminum = Vacuum;
  //Polycarbonate = Vacuum;

  //G4cout<<"test:d1"<<G4endl;
  // Print all the materials defined.                                                                                         
  //G4cout << G4endl << "The materials defined are : " << G4endl << G4endl;
  //G4cout << *(G4Material::GetMaterialTable()) << G4endl; 
  
  //===============================================Rotations=====================================================================================//                                                                
  G4RotationMatrix* zeroRotation = new G4RotationMatrix();
  G4RotationMatrix* xRotation = new G4RotationMatrix;
  G4double rotationAngleX = 90.0*deg;
  xRotation->rotateX(rotationAngleX);
  
  G4RotationMatrix* yRotation = new G4RotationMatrix; 
  G4double rotationAngleY = 90.0*deg;
  yRotation->rotateY(rotationAngleY);
  
  G4RotationMatrix* y2Rotation = new G4RotationMatrix; 
  G4double rotationAngle2Y = 90.0*deg;
  y2Rotation->rotateY(rotationAngle2Y);
  
  G4RotationMatrix* yFlip = new G4RotationMatrix; 
  G4double rotationAngleFlipY = 180.0*deg;
  yFlip->rotateY(rotationAngleFlipY);
  
  
  //=================================================Element Dimmensions=========================================================================//                                                                 
  G4double foilDepth  = thickness/2.0;

  G4ThreeVector doorDim = G4ThreeVector(1*inch/2, 15*inch/2, 17*inch/2);
  
  //Cubes
  G4double cubeThickness   = 0.5*inch; 
  G4ThreeVector cube1Dim((12/2)*inch + cubeThickness/2, (12/2)*inch + cubeThickness/2, (12/2)*inch + cubeThickness/2);
  G4ThreeVector cube2Dim(cube1Dim.x(), cube1Dim.y() - cubeThickness/2, cube1Dim.z() - cubeThickness/2);
  
  //Pipes
  G4double pipeLengthLong  = 2*inch;
  
  //Flanges
  G4double flangeDepth3 = 0.620*inch;

  //Plate
  G4ThreeVector plateDim((10.0/2)*inch, (0.375/2)*inch, (10.0/2)*inch);
  
  //Polarimeter
  G4double innerRadiusOfPolarimeter = 11*mm; 
  G4double outerRadiusOfPolarimeter = 35*mm; 
  G4double heightOfPolarimeter = 1.034*mm/2;
  //G4double heightOfPolarimeter = 0.3*mm/2;
  
  //PCB card
  G4ThreeVector cardDim(120*mm/2, 120*mm/2, 1.2*mm);
  
  //Rail Guide and Subtractions
  G4ThreeVector railGuideDim(9.25*inch/2, 0.988*inch/2, 0.625*inch/2);
  G4ThreeVector sub1Dim(10*inch/2, 0.365*inch/2, 0.375*inch/2);
  G4ThreeVector sub2Dim(10*inch/2, 1*inch/2, 0.125*inch/2);
  
  //Converter and Subtraction Shapes
  G4ThreeVector converterDim(5.25*inch/2, 5.792*inch/2, 3*mm/2);
  G4ThreeVector converterSubDim(4.325*inch/2, 4.292*inch/2, 4*mm/2);
  G4double converterTrayRadius       = 0.375*inch;
  
  //Motor
  G4ThreeVector motorPlateDim(42*mm/2, 42*mm/2, 2*mm/2);
  G4double motorLength                = (69/2.0)*mm;
  G4double gearRadius                 = (21/2.0)*mm;

  //================================Element positions============================================================================================// 
  G4ThreeVector positionWorld  = G4ThreeVector(0,0,0);
  G4ThreeVector posZero = G4ThreeVector(0,0,0);                                                                                  
  G4ThreeVector posChamber = G4ThreeVector(0,0,0);
  
  //Holes(Relative to chamber)
  G4ThreeVector posBeamHoleL  = G4ThreeVector(-1.5*inch,0.0*inch, cube1Dim.z() - cubeThickness/2.0) + posChamber;
  G4ThreeVector posBeamHoleR  = G4ThreeVector(-1.5*inch,0.0*inch, -cube1Dim.z() + cubeThickness/2.0) + posChamber;
  G4ThreeVector posMotorHole  = G4ThreeVector(posBeamHoleL.x() - 2.5*inch, posBeamHoleL.y() + 4.0*inch, posBeamHoleL.z());
  G4ThreeVector posTurboHole  = G4ThreeVector(0.0*inch, -cube1Dim.y() + cubeThickness/2.0, 0.0) + posChamber;
  G4ThreeVector posFeedHole   = G4ThreeVector(-cube1Dim.x() - cubeThickness/2.0, 0.0, -2.0*inch) + posChamber;
  G4ThreeVector posAuxHoleL   = G4ThreeVector(posBeamHoleR.x() + 3.5*inch, posBeamHoleR.y() + 3.0*inch, posBeamHoleL.z())+ posChamber;
  G4ThreeVector posAuxHoleR   = G4ThreeVector(posBeamHoleR.x() + 3.5*inch, posBeamHoleR.y() + 3.0*inch, -posBeamHoleL.z())+ posChamber;
  
  //Pipes(Relative to holes)
  G4ThreeVector posBeamPipeR  = G4ThreeVector(0.0,0.0,-pipeLengthLong/2.0 - cubeThickness/2.0) + posBeamHoleR;
  
  //Pipe flanges (Relative to pipes)
  G4ThreeVector posBeamFlangeR  = G4ThreeVector(0.0,0.0,flangeDepth3/2.0 - pipeLengthLong/2.0 + cubeThickness/4.0) + posBeamPipeR;
  
  //Door 
  G4ThreeVector posDoorFlange = G4ThreeVector(cube1Dim.x() -doorFlangeX, 0.0, 0.0) + posChamber;
  
  //Plate
  G4ThreeVector posPlate = G4ThreeVector(0.0 ,cube1Dim.y() -cubeThickness/2.0 -plateDim.y(), 0.0) + posChamber;
  
  //Polarimeter 
  G4ThreeVector positionPolarimeter  = G4ThreeVector(posBeamFlangeR.x(),0,posFeedHole.z());
  
  //Rail Guide (Relative to Plate)
  //G4ThreeVector posRailGuide  = G4ThreeVector(0.75/2*inch, -railGuideDim.y() -plateDim.y(),-35*mm + positionPolarimeter.z()) + posPlate;
  G4ThreeVector posRailGuide  = G4ThreeVector(0.75/2*inch, -railGuideDim.y() -plateDim.y(),-foilZ + positionPolarimeter.z()) + posPlate;
  G4ThreeVector posThinSub    = G4ThreeVector(0.0,-sub2Dim.y(),0.0) + posZero;
  
  //Converter Tray and Subtraction 
  G4ThreeVector posConverter     = G4ThreeVector(0.0,0.0,0.0) + posChamber;
  G4ThreeVector posConverterSub  = G4ThreeVector((0.925/2.0)*inch, (1.5/2.0)*inch,0.0) + posConverter;
  G4ThreeVector posTray1         = G4ThreeVector(converterDim.x() - 0.75*inch,-converterDim.y() + 0.75*inch,0.0) + posConverter;
  G4ThreeVector posTray2         = G4ThreeVector(-1.25*inch,0.0,0.0) + posTray1;
  G4ThreeVector posTray3         = G4ThreeVector(-1.25*inch,0.0,0.0) + posTray2;
                                                                           //**************VVVVV**********Converter Hole (0.75, 2.0, or 3.25)
  G4ThreeVector posConverterFin  = G4ThreeVector(positionPolarimeter.x()+converterDim.x() -0.75*inch,
						 positionPolarimeter.y() + converterDim.y() - 0.75*inch,
						 posRailGuide.z()) + posChamber;
  posFoil                        = G4ThreeVector(posBeamFlangeR.x(),0.0, posRailGuide.z());
  G4cout<<"(posPolZ - posfoilZ)/mm "<<(positionPolarimeter.z()-posFoil.z())/mm<<G4endl;
  //Motor 
  G4ThreeVector posMotor      = G4ThreeVector(positionPolarimeter.x(),
					      posRailGuide.y() -railGuideDim.y() -gearRadius -2*mm,
					      posRailGuide.z() + railGuideDim.z() + motorLength + motorPlateDim.z()) + posChamber;
  G4ThreeVector posMotorPlate = G4ThreeVector(0.0, 0.0, -motorLength);
  G4ThreeVector posGear       = G4ThreeVector(0.0, 0.0, -motorLength -2*motorPlateDim.z());
  
  //=====================================Make Solids=============================================================================================//
  
  solidWorld = new G4Box("world",24*inch,24*inch,24*inch);
  logicWorld = new G4LogicalVolume(solidWorld,Vacuum,"World",0,0,0);
  MakeShapes();
  
  G4Tubs *solidConverterFoil = new G4Tubs("foil",
					  0.0, 
					  converterTrayRadius,
					  foilDepth,
					  0.0*deg, 
					  360.0*deg);
  
  
  //====================================Subtract/Union Shapes====================================================================================//
  
  //Holes
  G4SubtractionSolid*  boxWithHolesTmp1 = new G4SubtractionSolid("boxWithHoles",solidShape[indexCube1],solidAddTube[2],zeroRotation,posBeamHoleL);
  G4SubtractionSolid*  boxWithHolesTmp2 = new G4SubtractionSolid("boxWithHolesTmp2",boxWithHolesTmp1,solidAddTube[2],zeroRotation,posBeamHoleR);
  G4SubtractionSolid*  boxWithHolesTmp3 = new G4SubtractionSolid("boxWithHolesTmp3",boxWithHolesTmp2,solidAddTube[3],zeroRotation,posMotorHole);
  G4SubtractionSolid*  boxWithHolesTmp4 = new G4SubtractionSolid("boxWithHolesTmp4",boxWithHolesTmp3,solidAddTube[6],xRotation,posTurboHole);
  G4SubtractionSolid*  boxWithHolesTmp5 = new G4SubtractionSolid("boxWithHolesTmp5",boxWithHolesTmp4,solidAddTube[4],zeroRotation,posAuxHoleL);
  G4SubtractionSolid*  boxWithHolesTmp6 = new G4SubtractionSolid("boxWithHolesTmp6",boxWithHolesTmp5,solidAddTube[4],zeroRotation,posAuxHoleR);
  G4SubtractionSolid*  boxWithHoles     = new G4SubtractionSolid("boxWithHoles",boxWithHolesTmp6,solidAddTube[5],yRotation,posFeedHole);
  
  //Cubes 1 and 2
  G4ThreeVector positionCube2            = G4ThreeVector(cubeThickness+0.001*inch,0*inch,0*inch);
  G4SubtractionSolid* solidBoxWithCutout = new G4SubtractionSolid("solidBoxWithCutout",boxWithHoles,solidShape[indexCube2],zeroRotation,positionCube2);
  
  //Door Flange
  G4SubtractionSolid* solidDoorFlange = new G4SubtractionSolid("solidBoxWithCutout",solidShape[indexDoorFlange],solidShape[indexCube2],zeroRotation,positionCube2);

  //Rail Guide
  G4UnionSolid*  solidGuideSubTmp = new G4UnionSolid("Guide",solidShape[indexRailGuideBox],solidShape[indexRailGuideThin],zeroRotation,posThinSub);
  G4SubtractionSolid*  solidRailGuide = new G4SubtractionSolid("solidRailGuide",solidShape[indexRailGuide],solidGuideSubTmp,zeroRotation,posZero);
  
  //Converter
  G4SubtractionSolid* solidConverterNoHole = new G4SubtractionSolid("s",solidShape[indexConverter],solidShape[indexConverterSub],zeroRotation,posConverterSub);
  G4SubtractionSolid* solidConverter1Hole  = new G4SubtractionSolid("s",solidConverterNoHole,solidAddTube[7],zeroRotation,posTray1);
  G4SubtractionSolid* solidConverter2Hole  = new G4SubtractionSolid("s",solidConverter1Hole,solidAddTube[7],zeroRotation,posTray2);
  G4SubtractionSolid* solidConverter       = new G4SubtractionSolid("s",solidConverter2Hole,solidAddTube[7],zeroRotation,posTray3);
  
  //Motor
  G4UnionSolid*  solidMotorWPlate = new G4UnionSolid("solidGuideTmp1",solidAddTube[0],solidShape[indexMotorPlate],zeroRotation,posMotorPlate);
  G4UnionSolid*  solidMotor       = new G4UnionSolid("solidGuideTmp1",solidMotorWPlate,solidAddTube[1],zeroRotation,posGear);
  
  //=================================Create logic ===============================================================================================//  
  G4LogicalVolume *logicBoxWithCutout = new G4LogicalVolume(solidBoxWithCutout,Iron,"logicTmp2");
  
  //Door
  G4LogicalVolume *logicDoorFlange = new G4LogicalVolume(solidDoorFlange,Iron,"logicTmp2");
  
  //Converter
  G4LogicalVolume *logicConverter = new G4LogicalVolume(solidConverter,Aluminum,"logicTmp2");
  //G4LogicalVolume *logicConverter = new G4LogicalVolume(solidConverter,Vacuum,"logicTmp2");

  //G4LogicalVolume *logicFoil      = new G4LogicalVolume(solidConverterFoil,Polycarbonate,"logicTmp2");
  G4LogicalVolume *logicFoil      = new G4LogicalVolume(solidConverterFoil,Beryllium,"logicTmp2");
  //G4LogicalVolume *logicFoil      = new G4LogicalVolume(solidConverterFoil,Vacuum,"logicTmp2");

  
  //Rail Guide
  G4LogicalVolume *logicRailGuide = new G4LogicalVolume(solidRailGuide,Aluminum,"logicTmp2");
  
  //Motor
  G4LogicalVolume *logicMotor = new G4LogicalVolume(solidMotor,Iron,"logicTmp2");
  
  //===============================Create and Place Physical=====================================================================================// 
  physiWorld = new G4PVPlacement(   0, // rotate matrix
				    positionWorld,     // at (x,y,z)
				    logicWorld,        // its logical volume
				    "World",           // its name
				    0,                 // its mother  volume
				    false,             // no boolean operations
				    0);                // copy number  
  MakeTube();
  MakeBoxes();
  
  //Box
  physiBoxWithCutout = new G4PVPlacement(   0,  
					    posChamber,      
					    logicBoxWithCutout,        
					    "Box",            
					    logicWorld,         
					    false,              
					    0); 
  
  //Door
  physiDoorFlange = new G4PVPlacement(   0,  
					 posDoorFlange,      
					 logicDoorFlange,        
					 "Box",            
					 logicWorld,         
					 false,              
					 0);
  
  //RailGuide
  physiRailGuide = new G4PVPlacement(   0,  
					posRailGuide,      
					logicRailGuide,        
					"Box",            
					logicWorld,         
					false,              
					0);
  
  //Converter
  physiConverter = new G4PVPlacement(   yFlip,  
					posConverterFin,      
					logicConverter,        
					"Box",            
					logicWorld,         
					false,              
					0); 
  physiFoil = new G4PVPlacement(    0,  
				    posFoil,      
				    logicFoil,        
				    "Foil",            
				    logicWorld,         
				    false,              
				    0); 
  //Motor
  physiMotor = new G4PVPlacement(   0,  
				    posMotor,      
				    logicMotor,        
				    "Box",            
				    logicWorld,         
				    false,              
				    0);  
  
  //========================================Polarimeter and Card=================================================================================//
  G4VisAttributes* PolVisAtt= new G4VisAttributes(G4Colour(0.0,1.0,1.0));
  
  G4double startAngleRing = 0.*deg;
  G4double spanningAngleRing = 11.25*deg;
  int maxRingNumber = 24; //Changed on 11/4/2013                                                                      
  int maxSectorNumber = 32; //Changed on 11/4/2013                                                      
  G4double deltaRing = (outerRadiusOfPolarimeter - innerRadiusOfPolarimeter)/maxRingNumber;
  G4double deltaPhi = spanningAngleRing;
  zPolFace = positionPolarimeter.z() - heightOfPolarimeter;
  zPolBack = positionPolarimeter.z() + heightOfPolarimeter;
  G4Tubs* solidRing; 
  G4LogicalVolume*   logicRing;
  G4VPhysicalVolume* physiRing;
  //int rsNum = -1;
  char ringName[80];
  for (int sectorID=0;sectorID < maxSectorNumber;sectorID++){
    G4double innerRadiusOfRing = innerRadiusOfPolarimeter;
    G4double outerRadiusOfRing = innerRadiusOfPolarimeter + deltaRing;

    for (int ringID=0;ringID < maxRingNumber;ringID++){
      //G4cout<<"(inner, outer R of ring)/mm = "<<innerRadiusOfRing<<" , "<<outerRadiusOfRing<<G4endl;
      //sprintf(ringName,"ringR%dS%d",ringID,sectorID);
      sprintf(ringName,"%d",(ringID + 1)*100 + (sectorID + 1));
      solidRing = new G4Tubs(ringName,
			     innerRadiusOfRing,
			     outerRadiusOfRing,
			     heightOfPolarimeter,
			     startAngleRing,
			     spanningAngleRing);
      logicRing = new G4LogicalVolume(solidRing,Silicon,ringName);
      physiRing = new G4PVPlacement(0,
				    positionPolarimeter,
				    logicRing,
				    ringName,
				    logicWorld,
				    false,
				    0);
      
      logicRing->SetVisAttributes(PolVisAtt);
      
      innerRadiusOfRing += deltaRing;
      outerRadiusOfRing += deltaRing;
    }
    
    //Set the next ring azimuthal angles                                                                              
    startAngleRing += deltaPhi;
  }
 
  //############
  G4double blankWidth1 = 0.6*um;
  solidBlank = new G4Tubs("blank",
			  innerRadiusOfPolarimeter,
			  outerRadiusOfPolarimeter,
			  blankWidth1/2.0,
			  0.*deg,
			  360.*deg);
  logicBlank = new G4LogicalVolume(solidBlank, Aluminum,"blank");
  G4double zBlank1 = positionPolarimeter.z() + heightOfPolarimeter + blankWidth1/2.0;
  G4ThreeVector positionBlank  = G4ThreeVector(posBeamFlangeR.x(),0,zBlank1);
  physiBlank = new G4PVPlacement(0,             // rotation matrix
				 positionBlank,  // at (x,y,z)
				 logicBlank,     // its logical volume
				 "blank",        // its name
				 logicWorld,  // its mother volume
				 false,       // no boolean operations
				 0);          // copy number
  
  G4double blankWidth2 = 3.5*um;
  solidBlank2 = new G4Tubs("blank2",
                       innerRadiusOfPolarimeter,
                       outerRadiusOfPolarimeter,
                       blankWidth2/2.0,
                       0.*deg,
                       360.*deg);
  logicBlank2 = new G4LogicalVolume(solidBlank2,Silicon,"blank2");
  G4double zBlank2 = zBlank1 + blankWidth1/2.0 + blankWidth2/2.0;
  G4ThreeVector positionBlank2  = G4ThreeVector(posBeamFlangeR.x(),0,zBlank2);
  physiBlank2 = new G4PVPlacement(0,             
				  positionBlank2,
				  logicBlank2,     
				  "blank2",        
				  logicWorld,  
				  false,       
				  0);          

  G4double blankWidth3 = 1.5*um;
  solidBlank3 = new G4Tubs("blank3",
                       innerRadiusOfPolarimeter,
                       outerRadiusOfPolarimeter,
                       blankWidth3/2.0,
                       0.*deg,
                       360.*deg);
  logicBlank3 = new G4LogicalVolume(solidBlank3,Aluminum,"blank3");
  G4double zBlank3 = zBlank2 + blankWidth2/2.0 + blankWidth3/2.0;
  G4ThreeVector positionBlank3  = G4ThreeVector(posBeamFlangeR.x(),0,zBlank3);
  physiBlank3 = new G4PVPlacement(0,          
				  positionBlank3,
				  logicBlank3,    
				  "blank3",       
				  logicWorld,  
				  false,       
				  0);         
  G4double blankWidth4 = 1.5*um;
  solidBlank4 = new G4Tubs("blank4",
			   innerRadiusOfPolarimeter,
			   outerRadiusOfPolarimeter,
			   blankWidth4/2.0,
			   0.*deg,
			   360.*deg);
  logicBlank4 = new G4LogicalVolume(solidBlank4,Aluminum,"blank4");
  //logicBlank4 = new G4LogicalVolume(solidBlank4,Vacuum,"blank4");
  G4double zBlank4 = positionPolarimeter.z() - heightOfPolarimeter - blankWidth4/2.0;
  G4ThreeVector positionBlank4  = G4ThreeVector(posBeamFlangeR.x(),0,zBlank4);
  physiBlank4 = new G4PVPlacement(0,            
				  positionBlank4,  
				  logicBlank4,     
				  "blank4",       
				  logicWorld,  
				  false,      
				  0);           


  G4double blankWidth5 = 0.5*mm;
  solidBlank5 = new G4Tubs("blank5",
			   0.0,
			   converterTrayRadius,
			   blankWidth5/2.0,
			   0.*deg,
			   360.*deg);

  //logicBlank5 = new G4LogicalVolume(solidBlank5,Polycarbonate,"blank5");
  logicBlank5 = new G4LogicalVolume(solidBlank5,Vacuum,"blank5");
  G4double zBlank5 = posFoil.z() + foilDepth + blankWidth5/2.0;
  G4ThreeVector positionBlank5  = G4ThreeVector(posBeamFlangeR.x(),0,zBlank5);
  physiBlank5 = new G4PVPlacement(0,            
				  positionBlank5,  
				  logicBlank5,     
				  "blank5",       
				  logicWorld,  
				  false,      
				  0);           

  //###########
  G4double zCardS3 = positionPolarimeter.z();
  G4double rTmp2 = 2.75*inch*sqrt(2.0);
  G4double innerRadiusOfHole = 0.0;
  G4double outerRadiusOfHole = outerRadiusOfPolarimeter;
  
  G4ThreeVector positionCardS3  = G4ThreeVector(0,0,zCardS3);
  G4ThreeVector positionTmp2Cut1  = G4ThreeVector(rTmp2,rTmp2,0);
  G4ThreeVector positionTmp2Cut2  = G4ThreeVector(-rTmp2,-rTmp2,0);
  G4ThreeVector positionTmp2Cut3  = G4ThreeVector(rTmp2,-rTmp2,0);
  G4ThreeVector positionTmp2Cut4  = G4ThreeVector(-rTmp2,rTmp2,0);
  
  G4Box  *solidCardS3Tmp = new G4Box("cardS3Tmp",
				     cardDim.x(),
				     cardDim.y(),
				     cardDim.z());
  G4Box *solidCardS3Tmp2 = new G4Box("cardS3Tmp2",
				     cardDim.x(),
				     cardDim.y(),
				     2.0*cardDim.z());
  
  G4Tubs *solidCardS3Hole = new G4Tubs("cardS3Hole",
				       innerRadiusOfHole,
				       outerRadiusOfHole,
				       2.0*cardDim.z(),
				       0.0*deg,
				       360.0*deg);
  
  G4SubtractionSolid *solidCardS3 = new G4SubtractionSolid("solidCardS3",solidCardS3Tmp,solidCardS3Hole, new G4RotationMatrix(),posZero);
  
  G4RotationMatrix* zRotation45 = new G4RotationMatrix;
  G4double rotationAngleZ = 45.0*deg;
  zRotation45->rotateZ(rotationAngleZ);
  solidCardS3 = new G4SubtractionSolid("solidCardS3",solidCardS3,solidCardS3Tmp2,zRotation45,positionTmp2Cut1);
  solidCardS3 = new G4SubtractionSolid("solidCardS3",solidCardS3,solidCardS3Tmp2,zRotation45,positionTmp2Cut2);
  solidCardS3 = new G4SubtractionSolid("solidCardS3",solidCardS3,solidCardS3Tmp2,zRotation45,positionTmp2Cut3);
  solidCardS3 = new G4SubtractionSolid("solidCardS3",solidCardS3,solidCardS3Tmp2,zRotation45,positionTmp2Cut4);
  
  G4ThreeVector positionCard = positionPolarimeter;

  G4LogicalVolume *logicCardS3 = new G4LogicalVolume(solidCardS3,Polycarbonate,"locicCardS3");
  physiCardS3 = new G4PVPlacement(0,              
				  positionCard,   
				  logicCardS3,     
				  "cardS3",         
				  logicWorld,      
				  false,            
				  0);               
  //=====================================Visualization attributes================================================================================//
  //Define Colors
  G4VisAttributes* WorldVisAtt= new G4VisAttributes(G4Colour(0.0,0.0,0.0)); logicWorld->SetVisAttributes(WorldVisAtt);
  G4VisAttributes* ConverterVisAtt= new G4VisAttributes(G4Colour(1.0,1.0,0.0)); ConverterVisAtt->SetForceSolid(true);
  G4VisAttributes* redSolidVA = new G4VisAttributes(G4Colour(1.0,0.0,0.0)); redSolidVA->SetForceSolid(true);
  G4VisAttributes* yellowSolidVA = new G4VisAttributes(G4Colour(1.0,1.0,0.0)); yellowSolidVA->SetForceSolid(true);
  G4VisAttributes* greenSolidVA = new G4VisAttributes(G4Colour(0.0,1.0,0.0)); greenSolidVA->SetForceSolid(true);
  G4VisAttributes* blueSolidVA = new G4VisAttributes(G4Colour(0.0,0.0,1.0)); blueSolidVA->SetForceSolid(true);
  G4VisAttributes* silverSolidVA = new G4VisAttributes(G4Colour(224.0/256,224.0/256,224.0/256)); silverSolidVA->SetForceSolid(true);
  G4VisAttributes* whiteSolidVA = new G4VisAttributes(G4Colour(1.0,1.0,1.0)); whiteSolidVA->SetForceSolid(true);
  G4VisAttributes* wireSolidVA = new G4VisAttributes(G4Colour(1.0,1.0,1.0));

  //Assign Colors
  logicCardS3->SetVisAttributes(yellowSolidVA); 
  logicConverter->SetVisAttributes(redSolidVA);          
  //logicConverter->SetVisAttributes(whiteSolidVA);          
  //logicFoil->SetVisAttributes(blueSolidVA);          
  logicBoxWithCutout->SetVisAttributes(silverSolidVA);    
  logicDoorFlange->SetVisAttributes(whiteSolidVA);    
  logicRailGuide->SetVisAttributes(whiteSolidVA);
  logicMotor->SetVisAttributes(whiteSolidVA);
  //logicBlank->SetVisAttributes(redSolidVA);
  //logicBlank2->SetVisAttributes(blueSolidVA);
  //logicBlank3->SetVisAttributes(redSolidVA);
  logicBlank4->SetVisAttributes(redSolidVA);
  //logicBlank5->SetVisAttributes(greenSolidVA);
  
  logicBlank->SetVisAttributes(G4VisAttributes::Invisible);
  logicBlank2->SetVisAttributes(G4VisAttributes::Invisible);
  logicBlank3->SetVisAttributes(G4VisAttributes::Invisible);
  //logicBlank4->SetVisAttributes(G4VisAttributes::Invisible);
  logicBlank5->SetVisAttributes(G4VisAttributes::Invisible);
  logicFoil->SetVisAttributes(G4VisAttributes::Invisible);
  //logicDoor->SetVisAttributes(G4VisAttributes::Invisible);

 for (unsigned int i = 0; i <= 13; i++) logicTube[i]->SetVisAttributes(silverSolidVA); //***Pipes and Flanges***//         
  for (unsigned int i = 14; i <= 17; i++) logicTube[i]->SetVisAttributes(whiteSolidVA); //***Flange Caps***//           
  logicRect[0]->SetVisAttributes(redSolidVA); //***Plate***//
  for (unsigned int i = 1; i <= 14 ; i++) logicRect[i]->SetVisAttributes(blueSolidVA); //***Supports and Detector Legs***//
  //logicRect[10]->SetVisAttributes(whiteSolidVA);
  logicRect[15]->SetVisAttributes(G4VisAttributes::Invisible);//***Door***//
  return physiWorld;
}
// ********************************************************************         
  
void myDetectorConstruction::SetRingSectorID(G4String volName)
{
  triPolRingID = -1;
  triPolSectorID = -1;
  
  int rsID = atoi(volName);
  int secNumTmp  = rsID % 100;
  int ringNumTmp = (rsID - secNumTmp)/100;
  if (secNumTmp >= 1 && secNumTmp <= 32)   triPolSectorID = secNumTmp -1; 
  if (ringNumTmp >= 1 && ringNumTmp <= 24) triPolRingID   = ringNumTmp -1; 
}
//*****Make Non-Physi Shapes(For Subtractions and Unions)
void myDetectorConstruction::MakeShapes()
{ 
  vector <G4ThreeVector> shapeDim; vector <G4ThreeVector> shapePos; vector <G4double> addDim; vector <G4double> lengths;
  G4double inch = 2.54*cm;
  G4double cubeThickness   = 0.5*inch; 
  G4double holeDepth = cubeThickness + 1.1*inch;
  G4double tubeThickness   = (1.0/16.0)*inch;
  
  //cube1
  indexCube1 = 0;
  G4ThreeVector cube1Dim((12/2)*inch + 0.5*inch/2, (12/2)*inch + 0.5*inch/2, (12/2)*inch + 0.5*inch/2);
  shapeDim.push_back(cube1Dim);
  
  //cube2
  indexCube2 = 1;
  G4ThreeVector cube2Dim(cube1Dim.x(), cube1Dim.y() - 0.5*inch/2, cube1Dim.z() - 0.5*inch/2);
  shapeDim.push_back(cube2Dim);
  
  //Door Flange Solid
  indexDoorFlange = 2;
  G4ThreeVector doorFlangeDim(0.5*inch/2,15*inch/2,17*inch/2);
  shapeDim.push_back(doorFlangeDim);
  
  //Rail Guide Solid
  indexRailGuide = 3;
  G4ThreeVector railGuideDim(9.25*inch/2, 0.988*inch/2, 0.625*inch/2);
  shapeDim.push_back(railGuideDim);
  
  //Rail Thin Sub Solid
  indexRailGuideThin = 4;
  G4ThreeVector sub2Dim(10*inch/2, 1*inch/2, 0.125*inch/2);
  shapeDim.push_back(sub2Dim);
  
  //Rail Box Sub Solid 
  indexRailGuideBox = 5;
  G4ThreeVector sub1Dim(10*inch/2, 0.365*inch/2, 0.375*inch/2);
  shapeDim.push_back(sub1Dim);
  
  //Motor Plate
  indexMotorPlate = 6;
  G4ThreeVector motorPlateDim(42*mm/2, 42*mm/2, 2*mm/2);
  shapeDim.push_back(motorPlateDim);
  
  //Converter Solid
  indexConverter = 7;
  G4ThreeVector converterDim(5.25*inch/2, 5.792*inch/2, 3*mm/2);
  shapeDim.push_back(converterDim);
  
  //Converter Subtraction Solid
  indexConverterSub = 8;
  G4ThreeVector converterSubDim(4.325*inch/2, 4.292*inch/2, 4*mm/2);
  shapeDim.push_back(converterSubDim);
  
  //Motor
  indexMotorTube = 0;
  G4double motorRadius = (42/2.0)*mm;
  G4double motorLength = (69/2.0)*mm;
  addDim.push_back(motorRadius); lengths.push_back(motorLength);
  
  //Gear
  indexMotorGear = 1;
  G4double gearRadius = (21/2.0)*mm;
  G4double gearLength = (32/2.0)*mm;
  addDim.push_back(gearRadius); lengths.push_back(gearLength);
  
  //Holes
  indexBeamHole = 2;
  G4double radiusBeamHole  = 1*inch - tubeThickness;
  addDim.push_back(radiusBeamHole); lengths.push_back(holeDepth);

  indexMotorHole = 3;
  G4double radiusMotorHole = 2.75*inch/2 - tubeThickness;
  addDim.push_back(radiusMotorHole); lengths.push_back(holeDepth);
  
  indexAuxHole = 4;
  G4double radiusAuxHole   = 1*inch -  tubeThickness;
  addDim.push_back(radiusAuxHole); lengths.push_back(holeDepth);

   indexFeedHole = 5;
  G4double radiusFeedHole  = 2*inch - tubeThickness;
  addDim.push_back(radiusFeedHole); lengths.push_back(holeDepth);

  indexTurboHole = 6;
  G4double radiusTurboHole = 2.75*inch/2 - tubeThickness;
  addDim.push_back(radiusTurboHole); lengths.push_back(holeDepth);

  indexConverterHole = 7;
  G4double converterTrayRadius = 0.375*inch;
  addDim.push_back(converterTrayRadius); lengths.push_back(holeDepth);

 for ( unsigned int i = 0; i <= 8; i++)
    solidShape.push_back(new G4Box("Box[i]",shapeDim[i].x(),shapeDim[i].y(), shapeDim[i].z())); 
  for ( unsigned int i = 0; i <= 7; i++)
    solidAddTube.push_back(new G4Tubs("Hole[i]", 0, addDim[i], lengths[i], 0, 360*degree));
}
//*****Make Cylindrical Phyisi Shapes (Pipes, Flanges, Caps)
void myDetectorConstruction::MakeTube()
{
  G4double inch = 2.54*cm;
  G4double cubeThickness   = 0.5*inch;
  G4ThreeVector posChamber = G4ThreeVector(0,0,0);
  G4ThreeVector cube1Dim((12/2)*inch + 0.5*inch/2, (12/2)*inch + 0.5*inch/2, (12/2)*inch + 0.5*inch/2);
 
  G4RotationMatrix* zeroRotation = new G4RotationMatrix();
  G4RotationMatrix* xRotation = new G4RotationMatrix;
  G4double rotationAngleX = 90.0*deg;
  xRotation->rotateX(rotationAngleX);
  G4RotationMatrix* yRotation = new G4RotationMatrix; 
  G4double rotationAngleY = 90.0*deg;
  yRotation->rotateY(rotationAngleY);

  G4ThreeVector posBeamHoleL  = G4ThreeVector(-1.5*inch,0.0*inch, cube1Dim.z() - cubeThickness/2.0) + posChamber;
  G4ThreeVector posBeamHoleR  = G4ThreeVector(-1.5*inch,0.0*inch, -cube1Dim.z() + cubeThickness/2.0) + posChamber;
  G4ThreeVector posMotorHole  = G4ThreeVector(posBeamHoleL.x() - 2.5*inch, posBeamHoleL.y() + 4.0*inch, posBeamHoleL.z());
  G4ThreeVector posTurboHole  = G4ThreeVector(0.0*inch, -cube1Dim.y() + cubeThickness/2.0, 0.0) + posChamber;
  G4ThreeVector posFeedHole   = G4ThreeVector(-cube1Dim.x() - cubeThickness/2.0, 0.0, -2.0*inch) + posChamber;
  G4ThreeVector posAuxHoleL   = G4ThreeVector(posBeamHoleR.x() + 3.5*inch, posBeamHoleR.y() + 3.0*inch, posBeamHoleL.z())+ posChamber;
  G4ThreeVector posAuxHoleR   = G4ThreeVector(posBeamHoleR.x() + 3.5*inch, posBeamHoleR.y() + 3.0*inch, -posBeamHoleL.z())+ posChamber;
  G4double pipeLengthLong  = 2*inch;
  G4double pipeLengthShort = 1.05*inch;
  G4double tubeThickness   = (1.0/16.0)*inch;
  G4double flangeDepth3 = 0.620*inch;
  G4double flangeDepth4 = 0.680*inch;
  G4double flangeDepth6 = 0.780*inch;
  G4NistManager* man = G4NistManager::Instance();
  G4Material* Iron      = man->FindOrBuildMaterial("G4_Fe");
  G4double atomicNumber = 1.;
  G4double massOfMole   = 1.008*g/mole;
  G4double density      = 1.e-25*g/cm3;
  G4double temperature  = 2.73*kelvin;
  G4double pressure=3.e-18*pascal; //SPACE                                                                                  
  G4Material *Vacuum =  new G4Material("Vacuum",atomicNumber,massOfMole,density,kStateGas,temperature,pressure);
  //ASDF
  //Iron = Vacuum;
  vector <G4ThreeVector> tubeDim; vector <G4ThreeVector> tubePos; vector <G4RotationMatrix*> tubeRot; vector <G4Material*> materials;

  //*****Dimension Vector Given as (Outer Radius, Inner Radius, Blank)
 
  //Beam Pipe R
  indexBeamPipeR  = 0; 
  G4ThreeVector beamPipeDim(1*inch, 1*inch -  tubeThickness, pipeLengthLong/2);
  G4ThreeVector posBeamPipeR  = G4ThreeVector(0.0,0.0,-pipeLengthLong/2.0 - cubeThickness/2.0) + posBeamHoleR; //Relative to Hole
  tubeDim.push_back(beamPipeDim); tubePos.push_back(posBeamPipeR); tubeRot.push_back(zeroRotation); tubeLength.push_back(beamPipeDim.z());
  G4Material *beamPipeMaterial = Iron; materials.push_back(beamPipeMaterial);

  //Beam Pipe L
  indexBeamPipeL  = 1;
  G4ThreeVector posBeamPipeL  = G4ThreeVector(0.0,0.0,pipeLengthLong/2.0 + cubeThickness/2.0) + posBeamHoleL; //Relative to Hole
  tubeDim.push_back(beamPipeDim); tubePos.push_back(posBeamPipeL); tubeRot.push_back(zeroRotation); tubeLength.push_back(beamPipeDim.z());
  materials.push_back(beamPipeMaterial);

  //Feed Pipe
  indexFeedPipe = 2;
  G4ThreeVector feedPipeDim(2*inch, 2*inch - tubeThickness, pipeLengthShort/2);
  G4ThreeVector posFeedPipe   = G4ThreeVector(-pipeLengthShort/2.0 + cubeThickness/2.0, 0.0, 0.0) + posFeedHole;  //Relative to Hole
  tubeDim.push_back(feedPipeDim); tubePos.push_back(posFeedPipe); tubeRot.push_back(yRotation); tubeLength.push_back(feedPipeDim.z());
  G4Material *feedPipeMaterial = Iron; materials.push_back(feedPipeMaterial);

  //Turbo Pipe
  indexTurboPipe = 3;
  G4ThreeVector turboPipeDim(2.75*inch/2, 2.75*inch/2 - tubeThickness, pipeLengthLong/2);
  G4ThreeVector posTurboPipe  = G4ThreeVector(0.0,-pipeLengthLong/2.0 - cubeThickness/2.0,0.0) + posTurboHole; //Relative to Hole
  tubeDim.push_back(turboPipeDim); tubePos.push_back(posTurboPipe); tubeRot.push_back(xRotation); tubeLength.push_back(turboPipeDim.z());
  G4Material *turboPipeMaterial = Iron; materials.push_back(turboPipeMaterial);
  
  //Motor Pipe
  indexMotorPipe = 4;
  G4ThreeVector motorPipeDim(turboPipeDim.x(), turboPipeDim.x() - tubeThickness, pipeLengthLong/2);
  G4ThreeVector posMotorPipe  = G4ThreeVector(0.0,0.0,pipeLengthLong/2.0 + cubeThickness/2.0) + posMotorHole; //Relative to Hole
  tubeDim.push_back(motorPipeDim); tubePos.push_back(posMotorPipe); tubeRot.push_back(zeroRotation); tubeLength.push_back(motorPipeDim.z());
  G4Material *motorPipeMaterial = Iron; materials.push_back(motorPipeMaterial);
  
  //Aux PipeR
  indexAuxPipeR = 5;
  G4ThreeVector auxPipeDim(beamPipeDim.x(), beamPipeDim.x() - tubeThickness, pipeLengthLong/2);
  G4ThreeVector posAuxPipeR   = G4ThreeVector(0.0,0.0,-pipeLengthLong/2.0 - cubeThickness/2.0) + posAuxHoleR; //Relative to Hole
  tubeDim.push_back(auxPipeDim); tubePos.push_back(posAuxPipeR); tubeRot.push_back(zeroRotation); tubeLength.push_back(auxPipeDim.z()); 
  G4Material *auxPipeMaterial = Iron; materials.push_back(auxPipeMaterial);

  //Aux PipeL
  indexAuxPipeL = 6;
  G4ThreeVector posAuxPipeL   = G4ThreeVector(0.0,0.0,pipeLengthLong/2 + cubeThickness/2) + posAuxHoleL; //Relative to Hole
  tubeDim.push_back(auxPipeDim); tubePos.push_back(posAuxPipeL); tubeRot.push_back(zeroRotation); tubeLength.push_back(auxPipeDim.z());
  materials.push_back(auxPipeMaterial);

  //Beam FlangeR
  indexBeamFlangeR = 7;
  G4ThreeVector beamFlangeDim(3.375*inch/2, beamPipeDim.x(), flangeDepth3/2);
  G4ThreeVector posBeamFlangeR  = G4ThreeVector(0.0,0.0,flangeDepth3/2 - pipeLengthLong/2) + posBeamPipeR; //Relative to Pipe
  tubeDim.push_back(beamFlangeDim); tubePos.push_back(posBeamFlangeR); tubeRot.push_back(zeroRotation); tubeLength.push_back(beamFlangeDim.z());
  G4Material *beamFlangeMaterial = Iron; materials.push_back(beamFlangeMaterial);

  //Beam FlangeL
  indexBeamFlangeL = 8 ;
  G4ThreeVector posBeamFlangeL  = G4ThreeVector(0.0,0.0,-flangeDepth3/2.0 + pipeLengthLong/2.0) + posBeamPipeL; //Relative to Pipe
  tubeDim.push_back(beamFlangeDim); tubePos.push_back(posBeamFlangeL); tubeRot.push_back(zeroRotation); tubeLength.push_back(beamFlangeDim.z());
  materials.push_back(beamFlangeMaterial);

  //Aux FlangeR
  indexAuxFlangeR =  9;  
  G4ThreeVector auxFlangeDim(3.375*inch/2, auxPipeDim.x(), flangeDepth3/2);
  G4ThreeVector posAuxFlangeR   = G4ThreeVector(0.0,0.0,flangeDepth3/2 - pipeLengthLong/2) + posAuxPipeR; //Relative to Pipe
  tubeDim.push_back(auxFlangeDim); tubePos.push_back(posAuxFlangeR); tubeRot.push_back(zeroRotation); tubeLength.push_back(auxFlangeDim.z());
  G4Material *auxFlangeMaterial = Iron; materials.push_back(auxFlangeMaterial);

  //AuxFlangeL
  indexAuxFlangeL = 10;
  G4ThreeVector posAuxFlangeL   = G4ThreeVector(0.0,0.0,-flangeDepth3/2 + pipeLengthLong/2) + posAuxPipeL; //Relative to Pipe
  tubeDim.push_back(auxFlangeDim); tubePos.push_back(posAuxFlangeL); tubeRot.push_back(zeroRotation); tubeLength.push_back(auxFlangeDim.z());
  materials.push_back(auxFlangeMaterial);

  //Motor Flange
  indexMotorFlange = 11;
  G4ThreeVector motorFlangeDim(4.5*inch/2, motorPipeDim.x(), flangeDepth4/2);
  G4ThreeVector posMotorFlange  = G4ThreeVector(0.0,0.0,-flangeDepth4/2 + pipeLengthLong/2) + posMotorPipe; //Relative to Pipe
  tubeDim.push_back(motorFlangeDim); tubePos.push_back(posMotorFlange); tubeRot.push_back(zeroRotation); tubeLength.push_back(motorFlangeDim.z());
  G4Material *motorFlangeMaterial = Iron; materials.push_back(motorFlangeMaterial);
  
  //Turbo Flange
  indexTurboFlange = 12;
  G4ThreeVector turboFlangeDim(4.5*inch/2, turboPipeDim.x(), flangeDepth4/2);
  G4ThreeVector posTurboFlange  = G4ThreeVector(0.0,flangeDepth4/2 - pipeLengthLong/2.0, 0.0) + posTurboPipe; //Relative to Pipe
  tubeDim.push_back(turboFlangeDim); tubePos.push_back(posTurboFlange); tubeRot.push_back(xRotation); tubeLength.push_back(turboFlangeDim.z());
  G4Material *turboFlangeMaterial = Iron; materials.push_back(turboFlangeMaterial);
  
  //Feed Flange
  indexFeedFlange = 13;
  G4ThreeVector feedFlangeDim(6*inch/2, feedPipeDim.x(), flangeDepth6/2);
  G4ThreeVector posFeedFlange   = G4ThreeVector(flangeDepth6/2 - pipeLengthShort/2,0.0,0.0) + posFeedPipe;  //Relative to Pipe 
  tubeDim.push_back(feedFlangeDim); tubePos.push_back(posFeedFlange); tubeRot.push_back(yRotation); tubeLength.push_back(feedFlangeDim.z());
  G4Material *feedFlangeMaterial = Iron; materials.push_back(feedFlangeMaterial);
  
  //AuxCapL
  indexAuxCapL = 14;
  G4ThreeVector posAuxCapL   = G4ThreeVector(0.0, 0.0, flangeDepth3/2 + 0.25*inch) + posAuxFlangeL; //Relative to Flange
  G4ThreeVector auxCapDim = G4ThreeVector(auxFlangeDim.x(),0,0);
  tubeDim.push_back(auxCapDim); tubePos.push_back(posAuxCapL); tubeRot.push_back(0); tubeLength.push_back(0.25*inch);
  G4Material *auxCapMaterial = Iron; materials.push_back(auxCapMaterial);
  
  //AuxCapR
  indexAuxCapR = 15;
  G4ThreeVector posAuxCapR   = G4ThreeVector(0.0, 0.0, -flangeDepth3/2 - 0.25*inch) + posAuxFlangeR; //Relative to Flange
  tubeDim.push_back(auxCapDim); tubePos.push_back(posAuxCapR); tubeRot.push_back(0); tubeLength.push_back(0.25*inch);
  materials.push_back(auxCapMaterial);

  //MotorCap
  indexMotorCap = 16;
  G4ThreeVector posMotorCap  = G4ThreeVector(0.0, 0.0, flangeDepth4/2 + 0.25*inch) + posMotorFlange; //Relative to Flange
  G4ThreeVector motorCapDim = G4ThreeVector(motorFlangeDim.x(), 0, 0);
  tubeDim.push_back(motorCapDim); tubePos.push_back(posMotorCap); tubeRot.push_back(0); tubeLength.push_back(0.25*inch);
  G4Material *motorCapMaterial = Iron; materials.push_back(motorCapMaterial);
  
  //FeedCap
  indexFeedCap = 17;
  G4ThreeVector posFeedCap   = G4ThreeVector(-flangeDepth6/2 - 0.25*inch, 0.0, 0.0) + posFeedFlange; //Relative to Flange
  G4ThreeVector feedCapDim   = G4ThreeVector(feedFlangeDim.x(),0,0);
  tubeDim.push_back(feedCapDim); tubePos.push_back(posFeedCap); tubeRot.push_back(yRotation); tubeLength.push_back(0.25*inch);
  G4Material *feedCapMaterial = Iron; materials.push_back(feedCapMaterial);
  
  for ( unsigned int i = 0; i <= 17; i++) {  
    solidTube.push_back(new G4Tubs("Tube[i]",tubeDim[i].y(),tubeDim[i].x(), tubeLength[i], 0, 360*degree));
    logicTube.push_back(new G4LogicalVolume(solidTube[i], materials[i], "tube[i]"));
    physiTube.push_back(new G4PVPlacement(tubeRot[i], tubePos[i], logicTube[i], "tube[i]", logicWorld, false, 0));
  }
  //G4cout<<"test:dend"<<G4endl;
}
//*****Make Physi Boxes(Plate, Plate Racks, Card Supports, Legs, and Door)
void myDetectorConstruction::MakeBoxes()
{   
  G4ThreeVector posChamber = G4ThreeVector(0,0,0);
  G4double cubeThickness   = 0.5*inch;
  G4double flangeDepth3 = 0.620*inch;
  G4double pipeLengthLong  = 2*inch;
  vector <G4ThreeVector> rectDim; vector <G4ThreeVector> rectPos; vector <G4RotationMatrix*> rectRot; vector <G4Material*> materials;
  
  G4RotationMatrix* y2Rotation = new G4RotationMatrix; 
  G4double rotationAngle2Y = 90.0*deg;
  y2Rotation->rotateY(rotationAngle2Y);

  G4ThreeVector cardDim(120*mm/2, 120*mm/2, 1.5*mm);
  G4ThreeVector cube1Dim(12*inch/2 + 0.5*inch/2, 12*inch/2 + 0.5*inch/2, 12*inch/2 + 0.5*inch/2);
  G4ThreeVector plateDim(10*inch/2, 0.375*inch/2, 10.0*inch/2); 
  G4ThreeVector posBeamHoleR  = G4ThreeVector(-1.5*inch,0.0*inch, -cube1Dim.z() + cubeThickness/2.0) + posChamber;
  G4ThreeVector posBeamPipeR  = G4ThreeVector(0.0,0.0,-pipeLengthLong/2.0 - cubeThickness/2.0) + posBeamHoleR;
  G4ThreeVector posFeedHole   = G4ThreeVector(-cube1Dim.x() - cubeThickness/2.0, 0.0, -2.0*inch) + posChamber;
  G4ThreeVector posBeamFlangeR  = G4ThreeVector(0.0,0.0,flangeDepth3/2.0 - pipeLengthLong/2.0 + cubeThickness/4.0) + posBeamPipeR;
  G4ThreeVector positionPolarimeter  = G4ThreeVector(posBeamFlangeR.x(),0,posFeedHole.z());
  
  G4NistManager* man = G4NistManager::Instance();
  G4Material* Iron      = man->FindOrBuildMaterial("G4_Fe");
  G4Material* Aluminum  = man->FindOrBuildMaterial("G4_Al");
  G4double atomicNumber = 1.;
  G4double massOfMole   = 1.008*g/mole;
  G4double density      = 1.e-25*g/cm3;
  G4double temperature  = 2.73*kelvin;
  G4double pressure=3.e-18*pascal; //SPACE                                                                                  
  G4Material *Vacuum =  new G4Material("Vacuum",atomicNumber,massOfMole,density,kStateGas,temperature,pressure);
  //ASDF
  //Aluminum = Vacuum;
  //Iron = Vacuum;
  //Plate
  indexPlate = 0;
  G4ThreeVector posPlate = G4ThreeVector(0.0, cube1Dim.y() -cubeThickness/2.0 -plateDim.y(), 0.0) + posChamber;
  rectDim.push_back(plateDim); rectPos.push_back(posPlate); rectRot.push_back(0);
  G4Material *plateMaterial = Aluminum; materials.push_back(plateMaterial);
  
  //VRackB
  indexVRackB = 1;
  G4ThreeVector vRackDim(6.0*inch/2, 0.51*inch/2, 0.25*inch/2);
  G4ThreeVector vPosRackB = G4ThreeVector(-vRackDim.z()-plateDim.x(),-vRackDim.y() + plateDim.y(),0.0) + posPlate; 
  rectDim.push_back(vRackDim); rectPos.push_back(vPosRackB); rectRot.push_back(y2Rotation);
  G4Material *rackMaterial = Iron; materials.push_back(rackMaterial);
  
  //VRackL
  indexVRackL = 2;
  G4ThreeVector vPosRackL = G4ThreeVector(0.0,-vRackDim.y() + plateDim.y(),-plateDim.z()-vRackDim.z()) + posPlate; 
  rectDim.push_back(vRackDim); rectPos.push_back(vPosRackL); rectRot.push_back(0); materials.push_back(rackMaterial);

  //HRackL
  indexHRackL = 3;
  G4ThreeVector hRackDim(6.0*inch/2, 0.125*inch/2, 0.75*inch/2);
  G4ThreeVector hPosRackL = G4ThreeVector(0.0,-vRackDim.y() + hRackDim.y(), vRackDim.z() + hRackDim.z() ) + vPosRackL;
  rectDim.push_back(hRackDim); rectPos.push_back(hPosRackL); rectRot.push_back(0); materials.push_back(rackMaterial);

  //VRackR
  indexVRackR = 4;
  G4ThreeVector vPosRackR = G4ThreeVector(0.0,-vRackDim.y() + plateDim.y(),+plateDim.z()+vRackDim.z()) + posPlate; 
  rectDim.push_back(vRackDim); rectPos.push_back(vPosRackR); rectRot.push_back(0); materials.push_back(rackMaterial);

  //HRackR
  indexHRackR = 5;
  G4ThreeVector hPosRackR = G4ThreeVector(0.0,-vRackDim.y() + hRackDim.y(), -1.0*(vRackDim.z() + hRackDim.z())) + vPosRackR; 
  rectDim.push_back(hRackDim); rectPos.push_back(hPosRackR); rectRot.push_back(0); materials.push_back(rackMaterial);

  //HRackB
  indexHRackB = 6;
  G4ThreeVector hPosRackB = G4ThreeVector(vRackDim.z() + hRackDim.z(), -vRackDim.y() + hRackDim.y(), 0.0) + vPosRackB; 
  rectDim.push_back(hRackDim); rectPos.push_back(hPosRackB); rectRot.push_back(y2Rotation); materials.push_back(rackMaterial);

  //HBarT
  indexHBarT = 7;
  G4ThreeVector barDim(4.75*inch/2, 0.5*inch/2, 0.125*inch/4);
  G4ThreeVector posBarHigh  = G4ThreeVector(0.0, cardDim.y() - barDim.y(), -cardDim.z() - barDim.z()) + positionPolarimeter;
  rectDim.push_back(barDim); rectPos.push_back(posBarHigh); rectRot.push_back(0);
   G4Material *barMaterial = Aluminum; materials.push_back(barMaterial);
 
  //HBarB
  indexHBarB = 8;
  G4ThreeVector posBarLow   = G4ThreeVector(0.0, -cardDim.y() + barDim.y(), -cardDim.z() - barDim.z()) + positionPolarimeter;
  rectDim.push_back(barDim); rectPos.push_back(posBarLow); rectRot.push_back(0); materials.push_back(barMaterial);

  //LegF
  indexLegF = 9;
  G4ThreeVector legDim(0.5*inch/2, 7.987*inch/2, 0.125*inch/2);
  G4ThreeVector posLegF  = G4ThreeVector(positionPolarimeter.x() + cardDim.x() - legDim.x(),-legDim.y() -plateDim.y() + posPlate.y(),positionPolarimeter.z() - 2*legDim.z() - cardDim.z());
  rectDim.push_back(legDim); rectPos.push_back(posLegF); rectRot.push_back(0);
  G4Material *legMaterial = Aluminum; materials.push_back(legMaterial);
  
  //LegB
  indexLegB = 10;
   G4ThreeVector posLegB  = G4ThreeVector(positionPolarimeter.x() - cardDim.x() + legDim.x(), -legDim.y() -plateDim.y() + posPlate.y(),positionPolarimeter.z() - 2*legDim.z() - cardDim.z());
  rectDim.push_back(legDim);rectPos.push_back(posLegB); rectRot.push_back(0); materials.push_back(legMaterial);

  //VSupportF
  indexVSupportF = 11;
  G4ThreeVector vSupportDim(0.5*inch/2, 2.5*inch/2, .25*inch/2);
  G4ThreeVector posVSupportF  = G4ThreeVector(0.0, legDim.y() - vSupportDim.y(), legDim.z() + vSupportDim.z()) + posLegF; 
  rectDim.push_back(vSupportDim); rectPos.push_back(posVSupportF); rectRot.push_back(0);
  G4Material *supportMaterial = Aluminum; materials.push_back(supportMaterial);
  
  //VSupportB
  indexVSupportB = 12;
  G4ThreeVector posVSupportB  = G4ThreeVector(0.0, legDim.y() - vSupportDim.y(), legDim.z() + vSupportDim.z()) + posLegB;
  rectDim.push_back(vSupportDim); rectPos.push_back(posVSupportB); rectRot.push_back(0); materials.push_back(supportMaterial);

  //HSupportF
  indexHSupportF = 13;
  G4ThreeVector hSupportDim(0.5*inch/2, 0.25*inch/2, 2.25*inch/2);
  G4ThreeVector posHSupportF  = G4ThreeVector(0.0, vSupportDim.y() - hSupportDim.y(), hSupportDim.z() + vSupportDim.z()) + posVSupportF;
  rectDim.push_back(hSupportDim); rectPos.push_back(posHSupportF); rectRot.push_back(0); materials.push_back(supportMaterial);

  //HSupportB
  indexHSupportB = 14;
  G4ThreeVector posHSupportB  = G4ThreeVector(0.0, vSupportDim.y() - hSupportDim.y(), hSupportDim.z() + vSupportDim.z()) + posVSupportB;
  rectDim.push_back(hSupportDim); rectPos.push_back(posHSupportB); rectRot.push_back(0); materials.push_back(supportMaterial);

  //Door
  indexDoor = 15;
  G4ThreeVector doorDim = G4ThreeVector(1*inch/2, 15*inch/2, 17*inch/2);
  G4ThreeVector posDoor       = G4ThreeVector(cube1Dim.x() + cubeThickness + .25*inch,0.0, 0.0) + posChamber;
  rectDim.push_back(doorDim); rectPos.push_back(posDoor); rectRot.push_back(0);
  G4Material *doorMaterial = Aluminum; materials.push_back(doorMaterial);
 
 for (unsigned int i = 0; i <= 15; i++) {
   char  nString[80];
   sprintf(nString,"RectPhyis%d",i);

    solidRect.push_back(new G4Box("Rectangle Component", rectDim[i].x(), rectDim[i].y(), rectDim[i].z()));
    logicRect.push_back(new G4LogicalVolume(solidRect[i], materials[i], "RectLog"));
    physiRect.push_back(new G4PVPlacement(rectRot[i], rectPos[i], logicRect[i],nString, logicWorld, false, 0));
  }
}
