//
// ********************************************************************
// *                                                                  *
// *                                                                  *
// *                                                                  *
// ********************************************************************
//
 
#include "myEventAction.hh"

#include "G4Event.hh"
#include "G4EventManager.hh"
#include "G4TrajectoryContainer.hh"
#include "G4Trajectory.hh"
#include "G4ios.hh"
#include "TH2.h"
#include "G4SDManager.hh"
#include "G4THitsCollection.hh"
#include "myPrimaryGeneratorAction.hh"
#include "TLorentzVector.h"
#include "TRandom3.h"
#include "TTree.h"
//#include "outTree.h"
#include "hList.hh"


//TH1D *hNRecoil;
//TH1D *hNDeltaRay;
//TH1D *hNBoth;
//TH1D *heDepNw;
//TH2D *heDepNwMult;
//TH1D *heDepNwTh;
//TH1D *heDepNwSmear12;
//TH1D *heDepNwSmear20;
//TH1D *heDepNwPart;
//TH1D *heDepNwSmear12Part;
//TH1D *heDepNwThPart;
//TH1D *heDepNwSmear20Part;
//TH2D *hAsymmetryDet1;
//TH2D *hAsymmetryDet2;
//TH1D *heDepNwSmear12Alpha;
//TH2D *heDepNwSmear12AlphaSec;
//TH1D *heDepNwSmear12PartAlpha;
//TH2D *hBeamXY;

class G4AttDef;

TRandom3 *randR = new TRandom3;

// ********************************************************************
void myEventAction::SetOutTree(TTree* treeToSet)
{
  outTree = treeToSet;
}
// ********************************************************************
myEventAction::myEventAction(myPrimaryGeneratorAction* pri)
 :primary(pri)
{ }
// ********************************************************************
myEventAction::~myEventAction()
{}
// ********************************************************************                                       
G4int myEventAction::GetNHits(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4int nHits = 0;
  for (G4int rID=0;rID<rIDmax;rID++) {
    for (G4int sID=0;sID<sIDmax;sID++) {
      if (ringEnergy[rID][sID]>0.0) {
        nHits += 1;
      }
    }
  }
  return nHits;
}
// ********************************************************************                                        
G4int myEventAction::GetNSectorHits(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4int nSectorHits = 0;
  G4int hit = -1;
  for (G4int sID=0;sID<sIDmax;sID++) {
    hit = -1;
    for (G4int rID=0;rID<rIDmax;rID++) {
      if (ringEnergy[rID][sID]>0.0) hit= 1;
    }
    if (hit ==1) nSectorHits += 1;
  }
  return nSectorHits;
}
// ********************************************************************                
G4int myEventAction::GetSector(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4int nSectorHits = 0;
  G4int hit = -1;
  G4int sectorID = -1;
  for (G4int sID=0;sID<sIDmax;sID++) {
    hit = -1;
    for (G4int rID=0;rID<rIDmax;rID++) {
      if (ringEnergy[rID][sID]>0.0){
        hit= 1;
        sectorID = sID;
      }
    }
    if (hit ==1) nSectorHits += 1;
  }
  return sectorID;
}
// ********************************************************************                
G4int myEventAction::GetSectorN(int hitN){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4int nSectorHits = 0;
  G4int hit = -1;
  G4int sectorID = -1;
  G4int sectorToGoID = -1;
  for (G4int sID=0;sID<sIDmax;sID++) {
    hit = -1;
    for (G4int rID=0;rID<rIDmax;rID++) {
      if (ringEnergy[rID][sID]>0.0){
        hit= 1;
        sectorID = sID;
      }
    }
    if (hit ==1) nSectorHits += 1;
    if (hitN == nSectorHits) sectorToGoID = sectorID;
  }
  return sectorToGoID;
}
// ********************************************************************                                        
void myEventAction::MakeSectorEnergy(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4double eDep = 0.0;
  for (G4int sID=0;sID<sIDmax;sID++) {
    eDep = 0.0;
    for (G4int rID=0;rID<rIDmax;rID++) {
      eDep += ringEnergy[rID][sID];
    }
    sectorEnergy[sID] = eDep;
  }
}
// ********************************************************************                                        
G4int myEventAction::GetRing(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4int nRingHits = 0;
  G4int hit = -1;
  G4int ringID = -1;
  for (G4int rID=0;rID<rIDmax;rID++) {
    hit = -1;
    for (G4int sID=0;sID<sIDmax;sID++) {
      if (ringEnergy[rID][sID]>0.0){
        hit= 1;
        ringID = rID;
      }
    }
    if (hit ==1) nRingHits += 1;
  }
  return ringID;
}
// ********************************************************************                                        
G4int myEventAction::GetNRingHits(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4int nRingHits = 0;
  G4int hit = -1;
  for (G4int rID=0;rID<rIDmax;rID++) {
    hit = -1;
    for (G4int sID=0;sID<sIDmax;sID++) {
      if (ringEnergy[rID][sID]>0.0) hit= 1;
    }
    if (hit ==1) nRingHits += 1;
  }
  return nRingHits;
}
// ********************************************************************                                        
G4double myEventAction::GetEdep(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4double eDep = 0.0;
  for (G4int rID=0;rID<rIDmax;rID++) {
    for (G4int sID=0;sID<sIDmax;sID++) {
      eDep += ringEnergy[rID][sID];
    }
  }
  return eDep;
}
// ********************************************************************                                        
G4double myEventAction::GetSectorEdep(int sector){
  G4int rIDmax = 24;
  G4double eDep = 0.0;
  for (G4int rID=0;rID<rIDmax;rID++) {
    eDep += ringEnergy[rID][sector];
  }
  return eDep;
}
// ********************************************************************                                                   
G4double myEventAction::GetEdepPart(){
  G4int rIDmax = 24;
  G4int sIDmax = 32;
  G4double eDep = 0.0;
  for (G4int rID=0;rID<rIDmax;rID++) {
    for (G4int sID=0;sID<sIDmax;sID++) {
      eDep += ringEnergyPart[rID][sID];
    }
  }
  return eDep;
}
// ********************************************************************                                        
G4double myEventAction::GetAngle(G4int sectorID){
  G4double angle, deltaAngle;
  deltaAngle = (360.0/32.0)*degree;
  angle = sectorID*deltaAngle + deltaAngle/2.0;
  return angle;
}
// ********************************************************************                             
int myEventAction::GetADC(G4double eDepIn){
  int fADCcount;
  double eDep = eDepIn/MeV;
  double calVal = 90; //mV per MeV
  double mVolts = calVal*eDep;

  double mV_per_fADCcount;
  mV_per_fADCcount = 1000.0/pow(2.0,12);

  fADCcount = mVolts/mV_per_fADCcount;

  return fADCcount;
}
// ********************************************************************                    
void myEventAction::BeginOfEventAction(const G4Event* evt)
{
  G4int event_id;
  event_id = evt->GetEventID();
  nRecoil = 0;
  nDeltaRay = 0;
  int rIDmax = 24;
  int sIDmax = 32;
  for (int rID=0;rID<rIDmax;rID++){
    for (int sID=0;sID<sIDmax;sID++){
      ringEnergy[rID][sID] = 0.0;
      ringEnergyPart[rID][sID] = 0.0;
    }
  }
}
// ********************************************************************
void myEventAction::EndOfEventAction(const G4Event* evt)
{
  //G4cout<<"test:e1"<<G4endl;

  extern struct mcHistos_t myHistos;


  G4int event_id = evt->GetEventID();

  G4TrajectoryContainer* trajectoryContainer = evt->GetTrajectoryContainer();
  G4int n_trajectories = 0;
  if (trajectoryContainer) n_trajectories = trajectoryContainer->entries();

  G4double weightXS = primary->GetWeightXS();
  G4double eGamma = primary->GetEgamma();
  G4double dePair = primary->GetDePair();
  G4double e1Val = primary->GetEpos();
  G4double e2Val = primary->GetEneg();
  G4double eRIn = primary->GetERIn();
  G4double mPair = primary->GetMpair();
  double cosR = primary->GetCosR();
  //G4cout<<"cosR = "<<cosR<<G4endl;
  G4double mPair2 = primary->GetMpair2();
  G4ThreeVector beamVert = primary->GetVertexTPOLCenter();
  G4double beamX = beamVert.x();
  G4double beamY = beamVert.y();
  
  myHistos.hBeamXY->Fill(beamX/mm,beamY/mm);
  //G4cout<<"beamX/mm, beamY/mm = "<<beamX/mm<<" , "<<beamY/mm<<G4endl;
  bool inPS = true;
  if (e1Val > 6.0*GeV) inPS = false;
  if (e2Val > 6.0*GeV) inPS = false;
  if (e1Val < 3.0*GeV) inPS = false;
  if (e2Val < 3.0*GeV) inPS = false;


  G4double e12Val = e1Val + e2Val;

  double resMeV20 = 0.020; //20 keV sigma smear                                                                       
  G4double eDepSmear20 = randR->Gaus(GetEdep()/MeV,resMeV20)*MeV;
  G4double eDepSmear20Part = randR->Gaus(GetEdepPart()/MeV,resMeV20)*MeV;

  double resMeV12 = 0.012; //12 keV sigma smear                                                                               
  G4double eDepSmear12 = randR->Gaus(GetEdep()/MeV,resMeV12)*MeV;
  G4double eDepSmear12Part = randR->Gaus(GetEdepPart()/MeV,resMeV12)*MeV;

  double resMeV10 = 0.010; //10 keV sigma smear                                                                               
  G4double eDepSmear10 = randR->Gaus(GetEdep()/MeV,resMeV10)*MeV;
  G4double eDepSmear10Part = randR->Gaus(GetEdepPart()/MeV,resMeV10)*MeV;

  double resMeV7 = 0.007; //12 keV sigma smear                                                                               
  G4double eDepSmear7 = randR->Gaus(GetEdep()/MeV,resMeV7)*MeV;
  G4double eDepSmear7Part = randR->Gaus(GetEdepPart()/MeV,resMeV7)*MeV;
    
  if (eDepSmear20 < 0.0) eDepSmear20 = 0.0;
  if (eDepSmear20Part < 0.0) eDepSmear20Part = 0.0;
  if (eDepSmear12Part < 0.0) eDepSmear12Part = 0.0;
  if (eDepSmear7Part < 0.0) eDepSmear7Part = 0.0;
  if (eDepSmear10Part < 0.0) eDepSmear10Part = 0.0;

  //Fill histograms
  myHistos.hNRecoil->Fill(1.0*GetNRecoil(),weightXS);
  myHistos.hNDeltaRay->Fill(1.0*GetNDeltaRay(),weightXS);
  if (GetNRecoil() > 0 && GetNDeltaRay() > 0){
    myHistos.hNBoth->Fill(1.0,weightXS);  
  } else {
    myHistos.hNBoth->Fill(0.0,weightXS);  
  }
  myHistos.heDepNwMult->Fill(1.0*GetNSectorHits(),GetEdep()/MeV);
  MakeSectorEnergy();
  for (int secI=0; secI < 32; secI++){
    myHistos.heDepNwSmear12AlphaSec->Fill(sectorEnergy[secI]/MeV,(secI+1)*1.0);
  }
  if (GetNSectorHits() == 1) {
    myHistos.heDepNw->Fill(GetEdep()/MeV);
    myHistos.heDepNwSmear20->Fill(eDepSmear20/MeV);
    myHistos.heDepNwSmear12->Fill(eDepSmear12/MeV);
    myHistos.heDepNwPart->Fill(GetEdepPart()/MeV);
    myHistos.heDepNwSmear20Part->Fill(eDepSmear20Part/MeV);
    myHistos.heDepNwSmear12Part->Fill(eDepSmear12Part/MeV);
    myHistos.heDepNwSmear12Alpha->Fill(eDepSmear12/MeV);
    myHistos.heDepNwSmear12PartAlpha->Fill(eDepSmear12Part/MeV);

    G4double threshold = randR->Gaus(120,12);
    //G4double threshold = randR->Gaus(1250-1044,12);
    //G4double threshold = randR->Gaus(1200-1050,12);
    //G4double threshold = randR->Gaus(130,13);//Run 153
    //G4double threshold = randR->Gaus(8.8*4,8.8*4/3.0);

    G4double smearToGo     = eDepSmear12;
    G4double smearToGoPart = eDepSmear12Part;
    //Threshold cut 
    int adcCount = GetADC(smearToGo);
    if (adcCount > 1.0*threshold) {
      myHistos.heDepNwTh->Fill(smearToGo/MeV);
    }

    //Threshold cut 
    adcCount = GetADC(smearToGoPart);
    if (adcCount > 1.0*threshold) {
      myHistos.heDepNwThPart->Fill(smearToGoPart/MeV);
    }
  }

  if (GetNSectorHits() == 1) {
    myHistos.hAsymmetryDet1->Fill(GetAngle(GetSector())/degree,weightXS);
  }



  if (GetNSectorHits() == 1 && inPS == true && 
      fabs(dePair/GeV) < 1.75 && GetEdep()/MeV > 0.25){ 
    //G4cout<<"inPS = "<<inPS<<G4endl;
    //G4cout<<"dePair = "<<fabs(dePair/GeV)<<G4endl;
    //G4cout<<"eDep = "<<GetEdep()/MeV<<G4endl;
    //G4cout<<"weightXS = "<<weightXS<<G4endl;
    //G4cout<<"GetAngle(GetSector()) = "<<GetAngle(GetSector())<<G4endl;

    myHistos.hAsymmetryDet2->Fill(GetAngle(GetSector())/degree,weightXS);

  }

  tpol1Ptr->nSectorHits = GetNSectorHits();
  for (int tIndex = 0; tIndex < tpol1Ptr->nSectorHits; tIndex++){
    int mcSec = GetSectorN(tIndex+1);
    tpol1Ptr->mcSector[tIndex] = mcSec;
    tpol1Ptr->phi[tIndex] = GetAngle(mcSec)/degree;
    tpol1Ptr->eDep[tIndex] = GetSectorEdep(mcSec)/MeV;
  }
  if (tpol1Ptr->nSectorHits > 0) {
    tpol1Ptr->eRIn = eRIn/GeV;
    tpol1Ptr->ePos = e1Val/GeV;
    tpol1Ptr->eNeg = e2Val/GeV;
    tpol1Ptr->weightXS = weightXS;
    tpol1Ptr->beamX = beamX/mm;
    tpol1Ptr->beamY = beamY/mm;
    tpol1Ptr->eGamma = eGamma/GeV;
    tpol1Ptr->mPair = mPair/GeV;
    tpol1Ptr->mPair2 = mPair2/GeV;
    tpol1Ptr->cosR = cosR;
    //G4cout<<"cosR = "<<tpol1Ptr->cosR<<G4endl;
    //std::cout<<"eRIn = "<<tpol1Ptr->eRIn<<std::endl;
    //G4cout<<"beamX/mm, beamY/mm = "<<beamX/mm<<" , "<<beamY/mm<<G4endl;
  }
  
  outTree->Fill();

  // periodic printing
  if (event_id%10000 == 0) {
  //if (event_id%100 == 0) {
    G4cout << ">>> Event " << evt->GetEventID() << G4endl;
  }
  //G4cout<<"test:ef"<<G4endl;
}

// ********************************************************************




