//
// ********************************************************************
// *                                                                  *
// *       Primary Generator (source of beam)                         *
// *                                                                  *
// ********************************************************************
//

#include "TTree.h"
#include "tripTreeStructs.hh"
#include "TH1.h"
#include "TH2.h"
#include "TLorentzVector.h"
#include "TVector3.h"
#include "devilTreePT.h"
#include <iostream>	
#include "hList.hh"


#include "myPrimaryGeneratorAction.hh"
#include "myDetectorConstruction.hh"
#include "myEventAction.hh"
#include "Randomize.hh"
#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "globals.hh"



//TH1D *h301all;
//TH1D *hFrac;
//TH1D *hDecayTest;
//TH2D *hErVsTheta;
//TH2D *hErVsThetaFull;
//TH2D *hq2VsThetaFull;
//TH2D *hm23VsThetaFull;
//TH2D *hbetaX2VsThetaFull;
//TH2D *hbetaXYVsThetaFull;
//TH2D *hbetaXZVsThetaFull;
//TH2D *hqXVsThetaFull;
//TH2D *hqX;
//TH2D *htheta23VsThetarFull;
//TH2D *hpCMVsThetaFull;
//TH2D *hlambdaVsThetaFull;
//TH2D *hAsymmetry0;
//TH2D *hAsymmetry1;
//TH2D *hAsymmetry2;

static int axisFlip = 0;

G4double myPrimaryGeneratorAction::GetDePair(){
  G4double ePos, eNeg, dePair;
  ePos = posVec.E()*GeV;
  eNeg = negVec.E()*GeV;
  dePair = ePos - eNeg;
  return dePair;
}

// ********************************************************************

myPrimaryGeneratorAction::myPrimaryGeneratorAction(myDetectorConstruction* det)
  :detector(det)
{ 
  int number_particle = 1;
  particleGun = new G4ParticleGun(number_particle);
}

// ********************************************************************

myPrimaryGeneratorAction::~myPrimaryGeneratorAction()
{
  delete particleGun;
}

// ********************************************************************

void myPrimaryGeneratorAction::SetGenTree(TTree* treeToSet)
{
  genTree = treeToSet;
}


void myPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{  
  //G4cout<<"T0:xOffset0/mm, yOffset0/mm = "<<xOffset0/mm<<" , "<<yOffset0/mm<<G4endl;
  //G4cout<<"test:p01"<<G4endl;  
  G4double mass;
  G4ParticleDefinition* particle;
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  devilTreePT_t trpGen;
  getBranchesT1(genTree,&trpGen);

  extern struct mcHistos_t myHistos;
  
  axisFlip++;
  if (axisFlip > 1) axisFlip = 0;
  //cout<<"axisFlip = "<<axisFlip<<endl;

  G4int i = anEvent->GetEventID();
  
  //G4double moX,moY,moZ;
  //G4ParticleMomentum moVec;
  genTree->GetEntry(i);
  
  //--> Get information of the recoil
  //moX = trpGen.recoilPx*GeV;
  //moY = trpGen.recoilPy*GeV;
  //moZ = trpGen.recoilPz*GeV;
  //moR = sqrt(moX*moX + moY*moY + moZ*moZ);
  
  eGamma = trpGen.eGamma*GeV;
  weightXS = trpGen.weight;
  double radToDeg = 180.0/3.14159;

  double rPx,rPy,rPz,rE,posPx,posPy,posPz,posE,negPx,negPy,negPz,negE;
  if (axisAlign == 1) axisFlip = 0;
  if (axisAlign == 2) axisFlip = 1;
  //if (axisFlip == 0) {
  rPx = trpGen.recoilPx;
  rPy = trpGen.recoilPy;
  rPz = trpGen.recoilPz;
  rE  = trpGen.recoilE;
  eR  = trpGen.recoilE*GeV;

  double rP = sqrt(pow(rPx,2)+pow(rPy,2)+pow(rPz,2));
  cosR = rPz/rP;
  //cout<<"eR = "<<eR/GeV<<endl;
  posPx = trpGen.positronPx;
  posPy = trpGen.positronPy;
  posPz = trpGen.positronPz;
  posE  = trpGen.positronE;
  
  negPx = trpGen.electronPx;
  negPy = trpGen.electronPy;
  negPz = trpGen.electronPz;
  negE  = trpGen.electronE;
  
  double pxPair = posPx + negPx;
  double pyPair = posPy + negPy;
  double pzPair = posPz + negPz;
  double ePair  = posE + negE;
  mPair = sqrt(pow(ePair,2)-pow(pxPair,2)-pow(pyPair,2)-pow(pzPair,2))*GeV;
  double pxPair2 = posPx + rPx;
  double pyPair2 = posPy + rPy;
  double pzPair2 = posPz + rPz;
  double ePair2 = trpGen.recoilE + posE;
  mPair2 = sqrt(pow(ePair2,2)-pow(pxPair2,2)-pow(pyPair2,2)-pow(pzPair2,2))*GeV;
  //}
  //G4cout<<"mPair/MeV = "<<mPair/MeV<<G4endl;
  if (axisFlip == 1) {
    rPx = trpGen.recoilPy;
    rPy = trpGen.recoilPx;
    rPz = trpGen.recoilPz;
    rE  = trpGen.recoilE;

    posPx = trpGen.positronPy;
    posPy = trpGen.positronPx;
    posPz = trpGen.positronPz;
    posE  = trpGen.positronE;

    negPx = trpGen.electronPy;
    negPy = trpGen.electronPx;
    negPz = trpGen.electronPz;
    negE  = trpGen.electronE;
  }
  
  //TLorentzVector recoilVec(trpGen.recoilPx,trpGen.recoilPy,trpGen.recoilPz,trpGen.recoilE);
  recVec3.set(rPx*GeV,rPy*GeV,rPz*GeV);
  TLorentzVector recoilVec(rPx,rPy,rPz,rE);
  posVec.SetPxPyPzE(posPx,posPy,posPz,posE);
  negVec.SetPxPyPzE(negPx,negPy,negPz,negE);
  TLorentzVector targetVec(0.0, 0.0, 0.0, 0.000511);
  
  G4ThreeVector pos3Vec(posPx,posPy,posPz);
  G4ThreeVector neg3Vec(negPx,negPy,negPz);

  TLorentzVector qVec   = targetVec - recoilVec; //note that q^2 = qVec.Mag2() and sqrt(q^2) = qVec.Mag()
  TLorentzVector kVec(0,0,eGamma, eGamma);
  TLorentzVector pVec23 = posVec + negVec;			  
  double e23 = pVec23.E();

  TLorentzVector lambdaVec = posVec - negVec;			  
  double kQVal = kVec*qVec;
  double kLambdaVal = kVec*lambdaVec;
  double fFrac = kLambdaVal/kQVal;

  double betaX = -qVec.Px()/e23;
  double betaY = -qVec.Py()/e23;
  double betaZ = (-eGamma -qVec.Pz())/e23;


  TVector3 betaVec(betaX, betaY, betaZ);
  G4double dePair = posVec.E()*GeV - negVec.E()*GeV;
  G4double e1Val = GetEpos();
  G4double e2Val = GetEneg();
  G4double e12Val = e1Val + e2Val;

  bool inPS = true;
  if (e1Val > 6.0*GeV) inPS = false;
  if (e2Val > 6.0*GeV) inPS = false;
  if (e1Val < 3.0*GeV) inPS = false;
  if (e2Val < 3.0*GeV) inPS = false;



  //  [0]*(1+[1]*cos(2*x*3.14159/180.0))
  //G4cout<<"test:p02"<<G4endl;  
  myHistos.hAsymmetry0->Fill(recoilVec.Phi()*radToDeg,weightXS);
  //if (fabs(dePair) < 1.75*GeV && inPS == true && e12Val >= 8.4*GeV && e12Val <= 9.2*GeV){
  if (inPS == true && e12Val >= 8.4*GeV && e12Val <= 9.2*GeV){
    myHistos.hAsymmetry1->Fill(recoilVec.Phi()*radToDeg,weightXS);
    if (17.45 <= recoilVec.Theta()*radToDeg && recoilVec.Theta()*radToDeg <= 45.0){
      myHistos.hAsymmetry2->Fill(recoilVec.Phi()*radToDeg,weightXS);
      myHistos.hFrac->Fill(fFrac,weightXS);
    }
  }
  //G4cout<<"test:p03"<<G4endl;  
  G4ThreeVector moRecoil(recoilVec.Px()*GeV,recoilVec.Py()*GeV,recoilVec.Pz()*GeV);
  rxnShot_t bullet, positron, electron;
  rxnDecay_t d1, chosenDecay;
  vector <rxnShot_t> loadVec;
 
  //Finding vertex position
  G4ThreeVector   posFoil = detector->GetPosFoil();
  G4double     converterZ = posFoil.z();
  G4double converterWidth = detector->GetConverterThickness();
  G4double     spotRadius = GetSpotRadius();
  G4double        xOffset = posFoil.x() + xOffset0;
  G4double        yOffset = posFoil.y() + yOffset0;
  G4double          zPos  = G4UniformRand()*converterWidth - converterWidth/2.0 + converterZ;
  bool             genPos = true;
  G4double          xPos = 0.0;
  G4double          yPos = 0.0;
  G4double         xyPos = 0.0;
  while (genPos == true) {
    xPos  = (G4UniformRand()*2.0*spotRadius - spotRadius);
    yPos  = (G4UniformRand()*2.0*spotRadius - spotRadius);
    xyPos  = sqrt(pow(xPos,2) + pow(yPos,2));
    if (xyPos <= spotRadius) genPos = false; 
  }
  vertexTPOLCenter.set(xPos+xOffset0,yPos+yOffset0,zPos);
  xPos += xOffset;
  yPos += yOffset;
  vertex.set(xPos,yPos,zPos);

  //G4cout<<"xPos/mm, yPos/mm = "<<xPos/mm<<" , "<<yPos/mm<<G4endl;
  //G4cout<<"xOffset0/mm, yOffset0/mm = "<<xOffset0/mm<<" , "<<yOffset0/mm<<G4endl;
 
 
  //Mode 1 = Recoil Only, Mode 2 = Pair Only, Mode 3 = Pair and Recoil , Mode 4 = decay Test

  //G4cout<<"test:p1"<<G4endl;
  //int mode = 3;
  //G4cout<<"test: mode = "<<mode<<G4endl;
  bool reverseFace = false;
  if (mode == 4) {
    //if (reverseFace == false) vertex.setZ(converterZ - 1*mm);//Keep vertex out of the converter!
    if (reverseFace == true) vertex.setZ(converterZ + 3.5*cm + 3.5*cm); //Set vertex to ring side
    double totDecays = 0;
    double runTot = 0;
    for (unsigned int i = 0; i < myDecayVec.size(); i++) totDecays += myDecayVec[i].decayPer100;
       
    //Pick Decays
    double randNum = G4UniformRand()*totDecays;
    bool foundDecay = false;
    int nTrue = 0;
    for (unsigned int i = 0; i < myDecayVec.size(); i++){
      runTot += myDecayVec[i].decayPer100;
      if (randNum < runTot && randNum >= runTot -  myDecayVec[i].decayPer100){
	chosenDecay = myDecayVec[i];
	foundDecay = true;
	nTrue++;
      }
      //G4cout<<"test:p2"<<G4endl;
    }
    //G4cout<<"test:p3"<<G4endl;
    if (randNum == totDecays){//Special case
      chosenDecay = myDecayVec[myDecayVec.size() - 1];
      foundDecay = true;
      //G4cout<<"test:p4"<<G4endl;
    };
    //G4cout<<"test:p5"<<G4endl;
    G4double energyToGenerate;
    if (chosenDecay.singleEnergy == true) {
      energyToGenerate = chosenDecay.eLow;
      
    }else{
      if (strcmp(chosenDecay.particleName,"e+") == 0 || strcmp(chosenDecay.particleName,"e-") == 0){
	double eCut = 0.0;
	int keepTest = 0;
	G4double me = 0.511*MeV;
	double feHigh = chosenDecay.eLow;
	while (keepTest == 0){
	  G4double Te = (chosenDecay.eHigh)*((1.0 - eCut)*G4UniformRand() + eCut);
	  G4double fe = sqrt(Te*Te + 2*Te*me)*pow(chosenDecay.eHigh - Te,2)*(Te + me);
	  if (fe > feHigh) G4cout<<"fe/MeV = "<<fe/MeV<<G4endl;
	  double feTest = G4UniformRand()*feHigh;
	  if (feTest <= fe){
	    keepTest = 1;
	    energyToGenerate = Te*MeV;
	  }
	}
      }else{
	energyToGenerate = G4UniformRand()*(chosenDecay.eHigh - chosenDecay.eLow) + chosenDecay.eLow; 
      }
    }
    G4String particleName = chosenDecay.particleName;
    //G4cout<<"test:p6"<<G4endl;       
    int alpha = strcmp(particleName, "alpha");
    if (alpha == 0){
      //particle = particleTable->GetIon(2,4,0.);
      //particleGun->SetParticleDefinition(particle);
    }else{
      particle = particleTable->FindParticle(chosenDecay.particleName);
    }
    //G4cout<<"test:p7"<<G4endl;
    mass = particle->GetPDGMass();
    //G4double thetaLow = 19*degree;
    //G4double thetaHigh = 44*degree;
    G4double thetaLow = 19*degree;
    G4double thetaHigh = 44*degree;
    if (reverseFace == true) { 
      thetaLow = (180 - 44)*degree;
      thetaHigh = (180 - 19)*degree;
      //thetaLow = 90*degree;
      //thetaHigh = 180*degree;
    } 
    //G4cout<<"thetaLow, thetaHigh = "<<thetaLow/degree<<" , "<<thetaHigh/degree<<G4endl;
    G4double thetaDecay = G4UniformRand()*(thetaHigh - thetaLow) + thetaLow;//detector angles
    G4double phiDecay   = G4UniformRand()*360.0*degree;
    G4double momentumDecay = sqrt(pow(energyToGenerate + mass,2) - pow(mass,2));
    double cx = sin(thetaDecay)*cos(phiDecay);    
    double cy = sin(thetaDecay)*sin(phiDecay);
    double cz = cos(thetaDecay);
    G4ThreeVector momentumToGenerate(momentumDecay*cx,momentumDecay*cy,momentumDecay*cz);
    if (foundDecay == true){
      myHistos.hDecayTest->Fill(energyToGenerate/MeV);
    }else{
      G4cout<<"!!!!!!!!!!!!!!!!!!!!!!!!BAD STUFF JUST HAPPENED!!!!!!!!!!!!!!!!!!!!!!!"<<G4endl;
    }
    bullet.particleName  = chosenDecay.particleName;
    bullet.energy        = energyToGenerate;
    bullet.momentum      = momentumToGenerate;
    bullet.position      = vertex;
    loadVec.push_back(bullet);
 
  }
  //G4cout<<"test:p8"<<G4endl;
  if (mode <= 3) {

    //if (mode == 1 || mode == 3) {
      //Load round(recoil) into vector
      bullet.particleName  = "e-";
      particle = particleTable->FindParticle(bullet.particleName);
      mass = particle->GetPDGMass();
      bullet.momentum      = moRecoil;
      bullet.position      = vertex;
      bullet.energy   = recoilVec.E()*GeV - mass;
      loadVec.push_back(bullet);
      //}
      //if (mode >= 2) {
      //Load round(electron) into vector
      electron.particleName = "e-";
      particle = particleTable->FindParticle(bullet.particleName);
      mass = particle->GetPDGMass();
      electron.energy  = trpGen.electronE*GeV - mass;
      electron.momentum     = neg3Vec;
      electron.position     = vertex;
      loadVec.push_back(electron);
      
      //Load round(positron) into vector
      positron.particleName = "e+";
      particle = particleTable->FindParticle(bullet.particleName);
      mass = particle->GetPDGMass();
      positron.energy  = trpGen.positronE*GeV - mass;
      positron.momentum     = pos3Vec;
      positron.position     = vertex;
      loadVec.push_back(positron);
      //}
  }

  int startIndex = 0; 
  int endIndex = loadVec.size();

  if (mode == 1) {
    startIndex = 0; 
    endIndex = 1; 
  } 
  if (mode == 2) {
    startIndex = 1; 
    endIndex = loadVec.size(); 
  } 
  if (mode == 3) { 
    startIndex = 0; 
    endIndex = loadVec.size(); 
  }

  //G4cout<<"test: startIndex endIndex = "<<startIndex<<" , "<<endIndex<<G4endl;

  //G4cout<<"test:p9"<<G4endl; 
  //for (unsigned int i = startIndex; i < endIndex; i++){
  for (int i = startIndex; i < endIndex; i++){
    //G4cout<<"test:p91"<<G4endl; 
    particle = particleTable->FindParticle(loadVec[i].particleName);
    //G4cout<<"test:p92"<<G4endl; 
    particleGun->SetParticleDefinition(particle);
    //G4cout<<"test:p93"<<G4endl; 
    particleGun->SetParticleEnergy(loadVec[i].energy);
    //G4cout<<"test:p94"<<G4endl; 
    particleGun->SetParticlePosition(loadVec[i].position);
    //G4cout<<"test:p95"<<G4endl; 
    particleGun->SetParticleMomentumDirection(loadVec[i].momentum/loadVec[i].momentum.mag());
    //G4cout<<"test:p96"<<G4endl; 
    particleGun->GeneratePrimaryVertex(anEvent);
    //G4cout<<"test:p97"<<G4endl; 
  }
}

// ********************************************************************

