


#include "TLorentzVector.h"
#include "hList.hh"

#include "mySteppingAction.hh"
#include "myDetectorConstruction.hh"
#include "myEventAction.hh"
#include "myPrimaryGeneratorAction.hh"
#include "G4Step.hh"
//#include "TH2.h"


#include "G4ElectronIonPair.hh"
#include "G4Material.hh"
#include "G4MaterialTable.hh"

//TH1D *hHit;
//TH1D *hHitSecondary;
//TH1D *hElectronKe;
//TH1D *hDeltaRayKe;
//TH2D *hElectronThetaKe;
//TH2D *hDeltaRayThetaKe;
//TH2D *hXY;
//TH2D *hDeltaThetaPhi;
//TH2D *hDeltaThetaKEr;
//TH2D *hDeltaPhiKEr;

mySteppingAction::mySteppingAction(myDetectorConstruction* det,
					   myEventAction* evt,
					   myPrimaryGeneratorAction* pri)
  :detector(det), eventaction(evt), primary(pri)
{ }


mySteppingAction::~mySteppingAction()
{ }

void mySteppingAction::UserSteppingAction(const G4Step* aStep)
{
  extern struct mcHistos_t myHistos;
  //G4cout<<"test:s1"<<G4endl;
 // get volume of the current step
  G4VPhysicalVolume* volume = aStep->GetPreStepPoint()->GetTouchableHandle()->GetVolume();

  G4double edep = aStep->GetTotalEnergyDeposit();

  G4StepPoint* prePoint = aStep->GetPreStepPoint();
  G4StepPoint* postPoint = aStep->GetPostStepPoint();
  //G4ThreeVector positionPost = postPoint->GetPosition();

  //Next get the positions of the pre- and post-step points:                                                                       
  G4ThreeVector pos1  = prePoint->GetPosition();
  G4ThreeVector pos2  = postPoint->GetPosition();
  G4double deltaZPol  = pos2.z() - detector->GetZPolFace();
  G4double deltaZPol2 = detector->GetZPolBack() - pos2.z();



  G4TouchableHandle touch1 = prePoint->GetTouchableHandle();

  //Find the volume of the pre and post-step points                             
  G4VPhysicalVolume* preVolume = touch1->GetVolume();
  G4String preVolumeName = preVolume->GetName();
  G4Track* p1track         = aStep->GetTrack();
  G4int parentId = p1track->GetParentID();
  G4ThreeVector p1momentum = p1track->GetMomentum();
  G4double ke = p1track->GetKineticEnergy();
  //G4cout<<"ke = "<<ke<<G4endl;
  //G4double thetaVal = p1momentum.theta();
  G4ParticleDefinition* outParticle = p1track->GetDefinition();
  //////////////
  detector->SetRingSectorID(volume->GetName());
  G4int triPolRingID = detector->GetRingID();
  G4int triPolSectorID = detector->GetSectorID();
  if (triPolRingID > -1 && triPolSectorID > -1) {

    eventaction->AddRingEnergy(edep,triPolRingID,triPolSectorID);
    //G4cout<<"eDep = "<<edep/MeV<<G4endl;
    //G4cout<<"getRingEnergy ["<<triPolRingID<<"] ["<<triPolSectorID<<"] = "<<
    //eventaction->GetRingEnergy(triPolRingID,triPolSectorID)/MeV<<G4endl;

    //if (deltaZPol > deadLow && deltaZPol2 > deadHigh){

    //G4cout<<"deadLow, deadHigh = "<<deadLow/um<<" , "<<deadHigh/um<<G4endl;
    if (deltaZPol > deadLow && deltaZPol2 > deadHigh){
      eventaction->AddRingEnergyPart(edep,triPolRingID,triPolSectorID);

      //G4cout<<"deltaZPol/um = "<<deltaZPol/um<<G4endl;
      //G4cout<<"pos1.z()/mm = "<<pos1.z()/mm<<G4endl;
      //G4cout<<"detector->GetZPolFace()/mm = "<<detector->GetZPolFace()/mm<<G4endl;
    }
  }
  if (prePoint->GetStepStatus() == fGeomBoundary && 
      volume->GetName() == "blank4" && p1momentum.z() > 0.0){
    G4ThreeVector vertVec = primary->GetVertex();
    G4ThreeVector rVec = primary->GetRecoilVec();
    G4double deltaXVal = pos1.x() - vertVec.x(); 
    G4double deltaYVal = pos1.y() - vertVec.y(); 
    G4double deltaZVal = pos1.z() - vertVec.z();
    G4ThreeVector deltaVec(deltaXVal,deltaYVal,deltaZVal);
    //G4double xyVal = sqrt(pow(pos1.x()+40.0*mm,2) + pow(pos1.y(),2));
    //G4double hypVal = sqrt(pow(xyVal,2) + pow(35.0*mm,2));
    //G4double cosVal = 35.0*mm/hypVal;
    //G4double thetaVal = acos(cosVal);
    G4double thetaVal = deltaVec.theta();
    if (parentId == 0) {

      myHistos.hElectronKe->Fill(ke/MeV,primary->GetWeightXS());
      myHistos.hElectronThetaKe->Fill(thetaVal/degree,ke/MeV,primary->GetWeightXS());
      myHistos.hXY->Fill(pos1.x()/mm + 40.0,pos1.y()/mm,primary->GetWeightXS());
      eventaction->AddNRecoil();
      G4double deltaTheta = deltaVec.theta() - rVec.theta();
      G4double deltaPhi = deltaVec.phi() - rVec.phi();
      G4double keR = primary->GetERIn() - 0.511*MeV;
      myHistos.hDeltaThetaPhi->Fill(deltaTheta/degree,deltaPhi/degree,primary->GetWeightXS());
      myHistos.hDeltaThetaKEr->Fill(deltaTheta/degree,keR/MeV,primary->GetWeightXS());
      myHistos.hDeltaPhiKEr->Fill(deltaPhi/degree,keR/MeV,primary->GetWeightXS());
      //if (deltaTheta/degree < -10.0){
	//G4cout<<"p1.mag, rVec.mag = "<<p1momentum.mag()/MeV<<" , "<<rVec.mag()/MeV<<G4endl;
      //}
    }
    if (parentId != 0 && outParticle->GetParticleName() == "e-") {
      myHistos.hDeltaRayKe->Fill(ke/MeV,primary->GetWeightXS());
      myHistos.hDeltaRayThetaKe->Fill(thetaVal/degree,ke/MeV,primary->GetWeightXS());
      eventaction->AddNDeltaRay();
    }
  } 
  
  //////////////

  //if (postPoint->GetStepStatus() == fGeomBoundary){
  //if (preVolumeName == "ringR%dS%d" && parentId == 0 ) {
      //  hHit->Fill(detector->GetConverterThickness()/um);
    //hElectronKe->Fill(ke,primary->GetWeightXS());
      //}
    //if (volume->GetName() == "ringR%dS%d") {
    //eventaction->AddMyEnergyPaddle1(edep);
    //}
    //if (preVolumeName == "ringR%dS%d" && parentId != 0 && outParticle->GetParticleName() == "e-") {
      //  hHitSecondary->Fill(detector->GetConverterThickness()/um);
    //}
  //}
  //G4cout<<"stepActionTest:END"<<G4endl;
  //G4cout<<"test:sf"<<G4endl;
}
